(* camlp5r *)
(* comp_head.ml.tpl,v *)
(* Copyright (c) INRIA 2007-2017 *)

#load "q_MLast.cmo";
#load "pa_extend.cmo";

module P =
  struct
    value gloc bp strm = Grammar.loc_of_token_interval bp (Stream.count strm);
    value vala symb = parser [: a = symb :] -> <:vala< a >>;
    value list0 symb =
      let rec loop al =
        parser
        [ [: a = symb; a = loop [a :: al] ! :] -> a
        | [: :] -> al ]
      in
      parser [: a = loop [] :] -> List.rev a
    ;
    value list0sep symb sep b =
      if not b then
        let rec kont al =
          parser
          [ [: v = sep; a = symb; a = kont [a :: al] ! :] -> a
          | [: :] -> al ]
        in
        parser
        [ [: a = symb; a = kont [a] ! :] -> List.rev a
        | [: :] -> [] ]
      else failwith "LIST0 _ SEP _ OPT_SEP not yet implemented"
    ;
    value list1 symb =
      let rec loop al =
        parser
        [ [: a = symb; a = loop [a :: al] ! :] -> a
        | [: :] -> al ]
      in
      parser [: a = symb; a = loop [a] ! :] -> List.rev a
    ;
    value list1sep symb sep b =
      if not b then
        let rec kont al =
          parser
          [ [: v = sep; a = symb; a = kont [a :: al] ! :] -> a
          | [: :] -> al ]
        in
        parser [: a = symb; a = kont [a] ! :] -> List.rev a
      else
        let rec kont al =
          parser
          [ [: v = sep; a = symb; a = kont [a :: al] ! :] -> a
          | [: v = sep :] -> al
          | [: :] -> al ]
        in
        parser [: a = symb; a = kont [a] ! :] -> List.rev a
    ;
    value option f =
      parser
      [ [: a = f :] -> Some a
      | [: :] -> None ]
    ;
    value bool f =
      parser
      [ [: _ = f :] -> True
      | [: :] -> False ]
    ;
    value token (p_con, p_prm) =
      if p_prm = "" then parser [: `(con, prm) when con = p_con :] -> prm
      else parser [: `(con, prm) when con = p_con && prm = p_prm :] -> prm
    ;
    value orzero f f0 =
      parser
      [ [: a = f :] -> a
      | [: a = f0 :] -> a ]
    ;
    value error entry prev_symb symb =
      symb ^ " expected" ^
      (if prev_symb = "" then "" else " after " ^ prev_symb) ^ " (in [" ^
      entry ^ "])"
    ;
    value lexer = lazy (Plexer.gmake ());
  end
;

(****************************************)

(* camlp5r *)
(* pa_o.ml,v *)
(* Copyright (c) INRIA 2007-2017 *)


open Pcaml;

Pcaml.syntax_name.val := "OCaml";
Pcaml.no_constructors_arity.val := True;

do {
  let odfa = Plexer.dollar_for_antiquotation.val in
  Plexer.dollar_for_antiquotation.val := False;
  Plexer.utf8_lexing.val := True;
  Grammar.Unsafe.gram_reinit gram (Lazy.force P.lexer);
  Plexer.dollar_for_antiquotation.val := odfa;
  Grammar.Unsafe.clear_entry interf;
  Grammar.Unsafe.clear_entry implem;
  Grammar.Unsafe.clear_entry top_phrase;
  Grammar.Unsafe.clear_entry use_file;
  Grammar.Unsafe.clear_entry module_type;
  Grammar.Unsafe.clear_entry module_expr;
  Grammar.Unsafe.clear_entry sig_item;
  Grammar.Unsafe.clear_entry str_item;
  Grammar.Unsafe.clear_entry signature;
  Grammar.Unsafe.clear_entry structure;
  Grammar.Unsafe.clear_entry expr;
  Grammar.Unsafe.clear_entry patt;
  Grammar.Unsafe.clear_entry ctyp;
  Grammar.Unsafe.clear_entry let_binding;
  Grammar.Unsafe.clear_entry type_decl;
  Grammar.Unsafe.clear_entry constructor_declaration;
  Grammar.Unsafe.clear_entry label_declaration;
  Grammar.Unsafe.clear_entry match_case;
  Grammar.Unsafe.clear_entry with_constr;
  Grammar.Unsafe.clear_entry poly_variant;
  Grammar.Unsafe.clear_entry class_type;
  Grammar.Unsafe.clear_entry class_expr;
  Grammar.Unsafe.clear_entry class_sig_item;
  Grammar.Unsafe.clear_entry class_str_item
};

Pcaml.parse_interf.val := Grammar.Entry.parse interf;
Pcaml.parse_implem.val := Grammar.Entry.parse implem;

value mklistexp loc last =
  loop True where rec loop top =
    fun
    [ [] ->
        match last with
        [ Some e -> e
        | None -> MLast.ExUid loc (Ploc.VaVal "[]") ]
    | [e1 :: el] ->
        let loc = if top then loc else Ploc.encl (MLast.loc_of_expr e1) loc in
        MLast.ExApp loc
          (MLast.ExApp loc (MLast.ExUid loc (Ploc.VaVal "::")) e1)
          (loop False el) ]
;

value mklistpat loc last =
  loop True where rec loop top =
    fun
    [ [] ->
        match last with
        [ Some p -> p
        | None -> MLast.PaUid loc (Ploc.VaVal "[]") ]
    | [p1 :: pl] ->
        let loc = if top then loc else Ploc.encl (MLast.loc_of_patt p1) loc in
        MLast.PaApp loc
          (MLast.PaApp loc (MLast.PaUid loc (Ploc.VaVal "::")) p1)
          (loop False pl) ]
;

value is_operator = do {
  let ht = Hashtbl.create 73 in
  let ct = Hashtbl.create 73 in
  List.iter (fun x -> Hashtbl.add ht x True)
    ["asr"; "land"; "lor"; "lsl"; "lsr"; "lxor"; "mod"; "or"];
  List.iter (fun x -> Hashtbl.add ct x True)
    ['!'; '&'; '*'; '+'; '-'; '/'; ':'; '<'; '='; '>'; '@'; '^'; '|'; '~';
     '?'; '%'; '.'; '$'];
  fun x ->
    try Hashtbl.find ht x with
    [ Not_found -> try Hashtbl.find ct x.[0] with _ -> False ]
};

value operator_rparen =
    (fun strm ->
       match Stream.npeek 2 strm with
       [ [("", s); ("", ")")] when is_operator s -> do {
           Stream.junk strm;
           Stream.junk strm;
           s
         }
       | _ -> raise Stream.Failure ])
;

value check_not_part_of_patt =
    (fun strm ->
       let tok =
         match Stream.npeek 4 strm with
         [ [("LIDENT", _); tok :: _] -> tok
         | [("", "("); ("", s); ("", ")"); tok] when is_operator s -> tok
         | _ -> raise Stream.Failure ]
       in
       match tok with
       [ ("", "," | "as" | "|" | "::") -> raise Stream.Failure
       | _ -> () ])
;

value symbolchar =
  let list =
    ['!'; '$'; '%'; '&'; '*'; '+'; '-'; '.'; '/'; ':'; '<'; '='; '>'; '?';
     '@'; '^'; '|'; '~']
  in
  loop where rec loop s i =
    if i == String.length s then True
    else if List.mem s.[i] list then loop s (i + 1)
    else False
;

value prefixop =
  let list = ['!'; '?'; '~'] in
  let excl = ["!="; "??"; "?!"] in
    (fun (strm__ : Stream.t _) ->
       match Stream.peek strm__ with
       [ Some ("", x)
         when
           not (List.mem x excl) && String.length x >= 2 &&
           List.mem x.[0] list && symbolchar x 1 -> do {
           Stream.junk strm__;
           x
         }
       | _ -> raise Stream.Failure ])
;

value infixop0 =
  let list = ['='; '<'; '>'; '|'; '&'; '$'] in
  let excl = ["<-"; "||"; "&&"] in
    (fun (strm__ : Stream.t _) ->
       match Stream.peek strm__ with
       [ Some ("", x)
         when
           not (List.mem x excl) && (x = "$" || String.length x >= 2) &&
           List.mem x.[0] list && symbolchar x 1 -> do {
           Stream.junk strm__;
           x
         }
       | _ -> raise Stream.Failure ])
;

value infixop1 =
  let list = ['@'; '^'] in
    (fun (strm__ : Stream.t _) ->
       match Stream.peek strm__ with
       [ Some ("", x)
         when
           String.length x >= 2 && List.mem x.[0] list &&
           symbolchar x 1 -> do {
           Stream.junk strm__;
           x
         }
       | _ -> raise Stream.Failure ])
;

value infixop2 =
  let list = ['+'; '-'] in
    (fun (strm__ : Stream.t _) ->
       match Stream.peek strm__ with
       [ Some ("", x)
         when
           x <> "->" && String.length x >= 2 && List.mem x.[0] list &&
           symbolchar x 1 -> do {
           Stream.junk strm__;
           x
         }
       | _ -> raise Stream.Failure ])
;

value infixop3 =
  let list = ['*'; '/'; '%'] in
    (fun (strm__ : Stream.t _) ->
       match Stream.peek strm__ with
       [ Some ("", x)
         when
           String.length x >= 2 && List.mem x.[0] list &&
           symbolchar x 1 -> do {
           Stream.junk strm__;
           x
         }
       | _ -> raise Stream.Failure ])
;

value infixop4 =
    (fun (strm__ : Stream.t _) ->
       match Stream.peek strm__ with
       [ Some ("", x)
         when
           String.length x >= 3 && x.[0] == '*' && x.[1] == '*' &&
           symbolchar x 2 -> do {
           Stream.junk strm__;
           x
         }
       | _ -> raise Stream.Failure ])
;

value test_constr_decl =
    (fun strm ->
       match Stream.npeek 1 strm with
       [ [("UIDENT", _)] ->
           match Stream.npeek 2 strm with
           [ [_; ("", ".")] -> raise Stream.Failure
           | [_; ("", "(")] -> raise Stream.Failure
           | [_ :: _] -> ()
           | _ -> raise Stream.Failure ]
       | [("", "|")] -> ()
       | _ -> raise Stream.Failure ])
;

value stream_peek_nth n strm =
  loop n (Stream.npeek n strm) where rec loop n =
    fun
    [ [] -> None
    | [x] -> if n == 1 then Some x else None
    | [_ :: l] -> loop (n - 1) l ]
;

(* horrible hack to be able to parse class_types *)

value test_ctyp_minusgreater =
    (fun strm ->
       let rec skip_simple_ctyp n =
         match stream_peek_nth n strm with
         [ Some ("", "->") -> n
         | Some ("", "[" | "[<") ->
             skip_simple_ctyp (ignore_upto "]" (n + 1) + 1)
         | Some ("", "(") -> skip_simple_ctyp (ignore_upto ")" (n + 1) + 1)
         | Some
             ("",
              "as" | "'" | ":" | "*" | "." | "#" | "<" | ">" | ".." | ";" |
              "_") ->
             skip_simple_ctyp (n + 1)
         | Some ("QUESTIONIDENT" | "LIDENT" | "UIDENT", _) ->
             skip_simple_ctyp (n + 1)
         | Some _ | None -> raise Stream.Failure ]
       and ignore_upto end_kwd n =
         match stream_peek_nth n strm with
         [ Some ("", prm) when prm = end_kwd -> n
         | Some ("", "[" | "[<") ->
             ignore_upto end_kwd (ignore_upto "]" (n + 1) + 1)
         | Some ("", "(") -> ignore_upto end_kwd (ignore_upto ")" (n + 1) + 1)
         | Some _ -> ignore_upto end_kwd (n + 1)
         | None -> raise Stream.Failure ]
       in
       match Stream.peek strm with
       [ Some (("", "[") | ("LIDENT" | "UIDENT", _)) -> skip_simple_ctyp 1
       | Some ("", "object") -> raise Stream.Failure
       | _ -> 1 ])
;

value test_label_eq =
    (test 1 where rec test lev strm =
       match stream_peek_nth lev strm with
       [ Some (("UIDENT", _) | ("LIDENT", _) | ("", ".")) ->
           test (lev + 1) strm
       | Some ("ANTIQUOT_LOC", _) -> ()
       | Some ("", "=" | ";" | "}") -> ()
       | _ -> raise Stream.Failure ])
;

value test_typevar_list_dot =
    (let rec test lev strm =
       match stream_peek_nth lev strm with
       [ Some ("", "'") -> test2 (lev + 1) strm
       | Some ("", ".") -> ()
       | _ -> raise Stream.Failure ]
     and test2 lev strm =
       match stream_peek_nth lev strm with
       [ Some ("UIDENT" | "LIDENT", _) -> test (lev + 1) strm
       | _ -> raise Stream.Failure ]
     in
     test 1)
;

value e_phony =
    (fun (strm__ : Stream.t _) -> raise Stream.Failure)
;
value p_phony =
    (fun (strm__ : Stream.t _) -> raise Stream.Failure)
;

value constr_arity = ref [("Some", 1); ("Match_Failure", 1)];

value rec is_expr_constr_call =
  fun
  [ MLast.ExUid _ (Ploc.VaVal _) -> True
  | MLast.ExAcc _ (MLast.ExUid _ (Ploc.VaVal _)) e -> is_expr_constr_call e
  | MLast.ExApp _ e _ -> is_expr_constr_call e
  | _ -> False ]
;

value rec constr_expr_arity loc =
  fun
  [ MLast.ExUid _ (Ploc.VaVal c) ->
      try List.assoc c constr_arity.val with [ Not_found -> 0 ]
  | MLast.ExAcc _ (MLast.ExUid _ (Ploc.VaVal _)) e -> constr_expr_arity loc e
  | _ -> 1 ]
;

value rec constr_patt_arity loc =
  fun
  [ MLast.PaUid _ (Ploc.VaVal c) ->
      try List.assoc c constr_arity.val with [ Not_found -> 0 ]
  | MLast.PaAcc _ (MLast.PaUid _ (Ploc.VaVal _)) p -> constr_patt_arity loc p
  | _ -> 1 ]
;

value get_seq =
  fun
  [ MLast.ExSeq _ (Ploc.VaVal el) -> el
  | e -> [e] ]
;

value mem_tvar s tpl =
  List.exists (fun (t, _) -> Pcaml.unvala t = Some s) tpl
;

value choose_tvar tpl =
  let rec find_alpha v =
    let s = String.make 1 v in
    if mem_tvar s tpl then
      if v = 'z' then None else find_alpha (Char.chr (Char.code v + 1))
    else Some (String.make 1 v)
  in
  let rec make_n n =
    let v = "a" ^ string_of_int n in
    if mem_tvar v tpl then make_n (succ n) else v
  in
  match find_alpha 'a' with
  [ Some x -> x
  | None -> make_n 1 ]
;

value quotation_content s =
  loop 0 where rec loop i =
    if i = String.length s then ("", s)
    else if s.[i] = ':' || s.[i] = '@' then
      let i = i + 1 in
      (String.sub s 0 i, String.sub s i (String.length s - i))
    else loop (i + 1)
;

value concat_comm loc e =
  let loc =
    Ploc.with_comment loc
      (Ploc.comment loc ^ Ploc.comment (MLast.loc_of_expr e))
  in
  let floc =
    let first = ref True in
    fun loc1 -> if first.val then do { first.val := False; loc } else loc1
  in
  Reloc.expr floc 0 e
;

value expr_of_patt p =
  let loc = MLast.loc_of_patt p in
  match p with
  [ MLast.PaLid _ (Ploc.VaVal x) -> MLast.ExLid loc (Ploc.VaVal x)
  | _ -> Ploc.raise loc (Stream.Error "identifier expected") ]
;


declare end;

(* Main entry points *)


Pcaml.add_option "-no_quot" (Arg.Set Plexer.no_quotations)
  "Don't parse quotations, allowing to use, e.g. \"<:>\" as token";
(* camlp5r *)
(* pa_op.ml,v *)
(* Copyright (c) INRIA 2007-2017 *)


open Exparser;
open Pcaml;

(* Syntax extensions in Ocaml grammar *)

value rec interf_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("", "#"); ("LIDENT", n)] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let dp =
            try P.option expr_0 strm__ with
            [ Stream.Failure -> raise (Stream.Error "") ]
          in
          match strm__ with parser
          [ [: `("", ";;") :] ->
              let loc = P.gloc bp strm__ in
              ([(MLast.SgDir loc (Ploc.VaVal n) (Ploc.VaVal dp), loc)], None)
          | [: :] -> raise (Stream.Error (P.error "interf" "" "';;'")) ]
        }
      | _ ->
          match strm__ with parser
          [ [: `("EOI", _) :] ->
              let loc = P.gloc bp strm__ in
              ([], Some loc)
          | [: si = sig_item_semi_0;
               (sil, stopped) = interf_0
                 ? P.error "interf" "[sig_item_semi]" "[interf]" :] ->
              ([si :: sil], stopped) ] ]
and implem_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("", "#"); ("LIDENT", n)] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let dp =
            try P.option expr_0 strm__ with
            [ Stream.Failure -> raise (Stream.Error "") ]
          in
          match strm__ with parser
          [ [: `("", ";;") :] ->
              let loc = P.gloc bp strm__ in
              ([(MLast.StDir loc (Ploc.VaVal n) (Ploc.VaVal dp), loc)], None)
          | [: :] -> raise (Stream.Error (P.error "implem" "" "';;'")) ]
        }
      | _ ->
          match strm__ with parser
          [ [: `("EOI", _) :] ->
              let loc = P.gloc bp strm__ in
              ([], Some loc)
          | [: si = str_item_semi_0;
               (sil, stopped) = implem_0
                 ? P.error "implem" "[str_item_semi]" "[implem]" :] ->
              ([si :: sil], stopped) ] ]
and top_phrase_0 =
  parser bp
  [ [: `("EOI", _) :] -> None
  | [: ph = phrase_0;
       `("", ";;") ? P.error "top_phrase" "[phrase]" "';;'" :] ->
      Some ph ]
and use_file_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("", "#"); ("LIDENT", n)] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let dp =
            try P.option expr_0 strm__ with
            [ Stream.Failure -> raise (Stream.Error "") ]
          in
          match strm__ with parser
          [ [: `("", ";;") :] ->
              let loc = P.gloc bp strm__ in
              ([MLast.StDir loc (Ploc.VaVal n) (Ploc.VaVal dp)], True)
          | [: :] -> raise (Stream.Error (P.error "use_file" "" "';;'")) ]
        }
      | _ ->
          match strm__ with parser
          [ [: `("EOI", _) :] -> ([], False)
          | [: si = str_item_0 :] ->
              let _ =
                try P.option (P.token ("", ";;")) strm__ with
                [ Stream.Failure -> raise (Stream.Error "") ]
              in
              match
                try Some (use_file_0 strm__) with [ Stream.Failure -> None ]
              with
              [ Some (sil, stopped) -> ([si :: sil], stopped)
              | _ ->
                  raise
                    (Stream.Error (P.error "use_file" "" "[use_file]")) ] ] ]
and phrase_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("", "#"); ("LIDENT", n)] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let dp =
            try P.option expr_0 strm__ with
            [ Stream.Failure -> raise (Stream.Error "") ]
          in
          let loc = P.gloc bp strm__ in
          MLast.StDir loc (Ploc.VaVal n) (Ploc.VaVal dp)
        }
      | _ ->
          match
            try Some (str_item_0 strm__) with [ Stream.Failure -> None ]
          with
          [ Some sti -> sti
          | _ -> raise Stream.Failure ] ]
and sig_item_semi_0 =
  parser bp
    [: si = sig_item_0 :] ->
      let _ =
        try P.option (P.token ("", ";;")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      let loc = P.gloc bp strm__ in
      (si, loc)
and expr_ident_0 =
  parser bp
  [ [: i = P.vala (P.token ("LIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExLid loc i
  | [: i = P.vala (P.token ("UIDENT", "")) :] ->
      try
        match strm__ with parser
        [ [: `("", ".") :] ->
            try
              match strm__ with parser
              [ [: `("", "(") :] ->
                  try
                    match
                      try Some (operator_rparen strm__) with
                      [ Stream.Failure -> None ]
                    with
                    [ Some j ->
                        let loc = P.gloc bp strm__ in
                        MLast.ExAcc loc (MLast.ExUid loc i)
                          (MLast.ExLid loc (Ploc.VaVal j))
                    | _ ->
                        match
                          try Some (expr_0 strm__) with
                          [ Stream.Failure -> None ]
                        with
                        [ Some e ->
                            match strm__ with parser
                            [ [: `("", ")") :] ->
                                let loc = P.gloc bp strm__ in
                                MLast.ExAre loc (MLast.ExUid loc i) e
                            | [: :] ->
                                raise
                                  (Stream.Error
                                     (P.error "expr_ident" "[expr]" "')'")) ]
                        | _ -> raise Stream.Failure ] ]
                  with
                  [ Stream.Failure ->
                      raise
                        (Stream.Error
                           (P.error "expr_ident" "'('"
                              "[operator_rparen] or [expr]")) ]
              | [: j = expr_ident_0 :] ->
                  let loc = P.gloc bp strm__ in
                  loop (MLast.ExUid loc i) j where rec loop m =
                    fun
                    [ MLast.ExAcc _ x y -> loop (MLast.ExAcc loc m x) y
                    | e -> MLast.ExAcc loc m e ] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "expr_ident" "'.'" "'(' or [expr_ident]")) ]
        | [: :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExUid loc i ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr_ident" "uppercase identifier" "'.'")) ] ]
and lbl_expr_0 =
  parser bp
    [: i = patt_label_ident_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             e = expr_1 ? P.error "lbl_expr" "'='" "[expr level expr1]" :] ->
            (i, e)
        | [: :] -> (i, expr_of_patt i) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "lbl_expr" "[patt_label_ident]" "'='")) ]
and lbl_expr_list_0 =
  parser bp
    [: le = lbl_expr_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (lbl_expr_list_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some lel -> [le :: lel]
              | _ -> [le] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "lbl_expr_list" "';'" "[lbl_expr_list]")) ]
        | [: :] -> [le] ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "lbl_expr_list" "[lbl_expr]" "';'")) ]
and expr1_semi_list_0 =
  parser bp [: el = P.list1sep expr_1 (P.token ("", ";")) True :] -> el
and label_0 = parser bp [: `("LIDENT", i) :] -> i
and field_expr_list_0 =
  parser bp
    [: l = label_0; `("", "=") ? P.error "field_expr_list" "[label]" "'='";
       e = expr_1 ? P.error "field_expr_list" "'='" "[expr level expr1]" :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (field_expr_list_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some fel -> [(l, e) :: fel]
              | _ -> [(l, e)] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "field_expr_list" "';'" "[field_expr_list]")) ]
        | [: :] -> [(l, e)] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "field_expr_list" "[expr level expr1]" "';'")) ]
and stream_expr_comp_0 =
  parser bp
  [ [: `("", "'");
       e = expr_1
         ? P.error "stream_expr_comp" "'''" "[expr level expr1]" :] ->
      let loc = P.gloc bp strm__ in
      SeTrm loc e
  | [: e = expr_1 :] ->
      let loc = P.gloc bp strm__ in
      SeNtr loc e ]
and stream_expr_comp_list_0 =
  parser bp
    [: se = stream_expr_comp_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (stream_expr_comp_list_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some sel -> [se :: sel]
              | _ -> [se] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "stream_expr_comp_list" "';'"
                        "[stream_expr_comp_list]")) ]
        | [: :] -> [se] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "stream_expr_comp_list" "[stream_expr_comp]" "';'")) ]
and direction_flag_0 =
  parser bp
  [ [: `("", "to") :] -> True
  | [: `("", "downto") :] -> False ]
and fun_def_0 =
  parser bp
  [ [: p = labeled_patt_0;
       (eo, e) = fun_def_0
         ? P.error "fun_def" "[labeled_patt]" "[fun_def]" :] ->
      let loc = P.gloc bp strm__ in
      (None, MLast.ExFun loc (Ploc.VaVal [(p, Ploc.VaVal eo, e)]))
  | [: p = patt_7;
       (eo, e) = fun_def_0
         ? P.error "fun_def" "[patt level simple]" "[fun_def]" :] ->
      let loc = P.gloc bp strm__ in
      (None, MLast.ExFun loc (Ploc.VaVal [(p, Ploc.VaVal eo, e)]))
  | [: :] ->
      let eo =
        try
          P.option
            (parser
               [: `("", "when");
                  e = expr_0 ? P.error "implem" "'when'" "[expr]" :] ->
                 e)
            strm__
        with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match strm__ with parser
      [ [: `("", "->"); e = expr_0 ? P.error "fun_def" "'->'" "[expr]" :] ->
          (eo, e)
      | [: :] -> raise (Stream.Error (P.error "fun_def" "" "'->'")) ] ]
and match_case_0 =
  parser bp
    [: x1 = patt_0;
       w =
         P.vala
           (P.option
              (parser
                 [: `("", "when");
                    e = expr_0 ? P.error "implem" "'when'" "[expr]" :] ->
                   e))
         ? P.error "match_case" "[patt]" "???";
       `("", "->") ? P.error "match_case" "???" "'->'";
       x2 = expr_0 ? P.error "match_case" "'->'" "[expr]" :] ->
      (x1, w, x2)
and stream_patt_let_0 =
  parser bp
    [: `("", "let");
       p = ipatt_0 ? P.error "stream_patt_let" "'let'" "[ipatt]";
       `("", "=") ? P.error "stream_patt_let" "[ipatt]" "'='";
       e = expr_0 ? P.error "stream_patt_let" "'='" "[expr]";
       `("", "in") ? P.error "stream_patt_let" "[expr]" "'in'" :] ->
      let loc = P.gloc bp strm__ in
      (SpLet loc p e, SpoNoth)
and stream_patt_comp_err_0 =
  parser bp
    [: spc = stream_patt_comp_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "??");
             e = expr_1
               ? P.error "stream_patt_comp_err" "'??'"
                   "[expr level expr1]" :] ->
            (spc, SpoQues e)
        | [: `("", "?!") :] -> (spc, SpoBang)
        | [: :] -> (spc, SpoNoth) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "stream_patt_comp_err" "[stream_patt_comp]"
                  "'??' or '?!'")) ]
and stream_patt_kont_0 =
  parser bp
  [ [: spc = stream_patt_comp_err_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (stream_patt_kont_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some sp -> [spc :: sp]
              | _ -> [spc] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "stream_patt_kont" "';'"
                        "[stream_patt_kont]")) ]
        | [: :] -> [spc] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "stream_patt_kont" "[stream_patt_comp_err]" "';'")) ]
  | [: spc = stream_patt_let_0;
       sp = stream_patt_kont_0
         ? P.error "stream_patt_kont" "[stream_patt_let]"
             "[stream_patt_kont]" :] ->
      [spc :: sp] ]
and lookahead_0 =
  parser bp
    [: `("", "[");
       pl = P.list1sep patt_0 (P.token ("", ";")) False
         ? P.error "lookahead" "'['" "[patt]";
       `("", "]") ? P.error "lookahead" "" "[patt] or ']'" :] ->
      pl
and mod_type_ident_0 =
  parser bp
    [: :] ->
      let a = mod_type_ident_1 strm__ in
      mod_type_ident_gen_cont bp a strm__
and mod_type_ident_0_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       m2 = P.orzero mod_type_ident_1 mod_type_ident_0
         ? P.error "mod_type_ident" "'.'" "[mod_type_ident]" :] ->
      let loc = P.gloc bp strm__ in
      let m1 = a__ in
      mod_type_ident_0_cont bp (MLast.MtAcc loc m1 m2) strm__
  | [: `("", "(");
       m2 = mod_type_ident_0
         ? P.error "mod_type_ident" "'('" "[mod_type_ident]";
       `("", ")") ? P.error "mod_type_ident" "[mod_type_ident]" "')'" :] ->
      let loc = P.gloc bp strm__ in
      let m1 = a__ in
      mod_type_ident_0_cont bp (MLast.MtApp loc m1 m2) strm__
  | [: :] -> a__ ]
and mod_type_ident_1 =
  parser bp
  [ [: m = P.vala (P.token ("UIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.MtUid loc m
  | [: m = P.vala (P.token ("LIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.MtLid loc m ]
and mod_type_ident_gen_cont bp a__ strm__ =
  let x = mod_type_ident_0_cont bp a__ strm__ in
  if x == a__ then x else mod_type_ident_gen_cont bp x strm__
and module_declaration_0 =
  parser bp
  [ [: `("", ":");
       mt = module_type_0
         ? P.error "module_declaration" "':'" "[module_type]" :] ->
      mt
  | [: arg = P.vala functor_parameter_0;
       mt = module_declaration_0
         ? P.error "module_declaration" "[functor_parameter]"
             "[module_declaration]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MtFun loc arg mt ]
and mod_decl_binding_0 =
  parser bp
    [: i = P.vala uidopt_0;
       mt = module_declaration_0
         ? P.error "mod_decl_binding" "[uidopt]" "[module_declaration]" :] ->
      (i, mt)
and class_description_0 =
  parser bp
    [: vf = P.vala (P.bool (P.token ("", "virtual")));
       ctp = class_type_parameters_0
         ? P.error "class_description" "???" "[class_type_parameters]" :] ->
      match Stream.npeek 2 strm__ with
      [ [("LIDENT", n); ("", ":")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let n = Ploc.VaVal n in
          match
            try Some (class_type_0 strm__) with [ Stream.Failure -> None ]
          with
          [ Some ct ->
              let loc = P.gloc bp strm__ in
              {MLast.ciLoc = loc; MLast.ciVir = vf; MLast.ciPrm = ctp;
               MLast.ciNam = n; MLast.ciExp = ct}
          | _ ->
              raise
                (Stream.Error
                   (P.error "class_description" "lowercase identifier"
                      "[class_type]")) ]
        }
      | _ ->
          raise
            (Stream.Error
               (P.error "class_description" "[class_type_parameters]"
                  "lowercase identifier")) ]
and sig_item_0 =
  parser bp
  [ [: `("", "class") :] ->
      try
        match strm__ with parser
        [ [: `("", "type");
             ctd =
               P.vala
                 (P.list1sep class_type_declaration_0 (P.token ("", "and"))
                    False)
               ? P.error "sig_item" "'type'" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.SgClt loc ctd
        | [: cd =
               P.vala
                 (P.list1sep class_description_0 (P.token ("", "and"))
                    False) :] ->
            let loc = P.gloc bp strm__ in
            MLast.SgCls loc cd ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "sig_item" "'class'" "'type' or ???")) ]
  | [: `("", "exception");
       (_, c, tl, _) = constructor_declaration_0
         ? P.error "sig_item" "'exception'" "[constructor_declaration]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.SgExc loc c tl
  | [: `("", "external") :] ->
      try
        match strm__ with parser
        [ [: `("", "(");
             i = operator_rparen
               ? P.error "sig_item" "'('" "[operator_rparen]";
             `("", ":") ? P.error "sig_item" "[operator_rparen]" "':'";
             t = ctyp_0 ? P.error "sig_item" "':'" "[ctyp]";
             `("", "=") ? P.error "sig_item" "[ctyp]" "'='";
             pd = P.vala (P.list1 (P.token ("STRING", "")))
               ? P.error "sig_item" "'='" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.SgExt loc (Ploc.VaVal i) t pd
        | [: :] ->
            match Stream.npeek 2 strm__ with
            [ [("LIDENT", i); ("", ":")] -> do {
                Stream.junk strm__;
                Stream.junk strm__;
                let i = Ploc.VaVal i in
                match
                  try Some (ctyp_0 strm__) with [ Stream.Failure -> None ]
                with
                [ Some t ->
                    match strm__ with parser
                    [ [: `("", "=");
                         pd = P.vala (P.list1 (P.token ("STRING", "")))
                           ? P.error "sig_item" "'='" "???" :] ->
                        let loc = P.gloc bp strm__ in
                        MLast.SgExt loc i t pd
                    | [: :] ->
                        raise
                          (Stream.Error (P.error "sig_item" "[ctyp]" "'='")) ]
                | _ ->
                    raise
                      (Stream.Error
                         (P.error "sig_item" "lowercase identifier"
                            "[ctyp]")) ]
              }
            | _ -> raise Stream.Failure ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "sig_item" "'external'"
                  "'(' or lowercase identifier")) ]
  | [: `("", "include");
       mt = module_type_0
         ? P.error "sig_item" "'include'" "[module_type]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.SgInc loc mt
  | [: `("", "module") :] ->
      try
        match strm__ with parser
        [ [: `("", "type");
             i = P.vala ident_0 ? P.error "sig_item" "'type'" "[ident]" :] ->
            try
              match strm__ with parser
              [ [: `("", "=");
                   mt = module_type_0
                     ? P.error "sig_item" "'='" "[module_type]" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.SgMty loc i mt
              | [: :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.SgMty loc i
                    (MLast.MtQuo loc (Ploc.VaVal "abstract")) ]
            with
            [ Stream.Failure ->
                raise (Stream.Error (P.error "sig_item" "[ident]" "'='")) ]
        | [: rf = P.vala (P.bool (P.token ("", "rec")));
             l =
               P.vala
                 (P.list1sep mod_decl_binding_0 (P.token ("", "and")) False)
               ? P.error "sig_item" "???" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.SgMod loc rf l ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "sig_item" "'module'" "'type' or ???")) ]
  | [: `("", "open");
       i = P.vala mod_ident_0
         ? P.error "sig_item" "'open'" "[mod_ident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.SgOpn loc i
  | [: `("", "type");
       tdl =
         P.vala (P.list1sep type_declaration_0 (P.token ("", "and")) False)
         ? P.error "sig_item" "'type'" "???" :] ->
      let loc = P.gloc bp strm__ in
      MLast.SgTyp loc tdl
  | [: `("", "val") :] ->
      try
        match strm__ with parser
        [ [: `("", "(");
             i = operator_rparen
               ? P.error "sig_item" "'('" "[operator_rparen]";
             `("", ":") ? P.error "sig_item" "[operator_rparen]" "':'";
             t = ctyp_0 ? P.error "sig_item" "':'" "[ctyp]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.SgVal loc (Ploc.VaVal i) t
        | [: :] ->
            match Stream.npeek 2 strm__ with
            [ [("LIDENT", i); ("", ":")] -> do {
                Stream.junk strm__;
                Stream.junk strm__;
                let i = Ploc.VaVal i in
                match
                  try Some (ctyp_0 strm__) with [ Stream.Failure -> None ]
                with
                [ Some t ->
                    let loc = P.gloc bp strm__ in
                    MLast.SgVal loc i t
                | _ ->
                    raise
                      (Stream.Error
                         (P.error "sig_item" "lowercase identifier"
                            "[ctyp]")) ]
              }
            | _ -> raise Stream.Failure ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "sig_item" "'val'" "'(' or lowercase identifier")) ] ]
and signature_0 =
  parser bp
    [: sg =
         P.vala
           (P.list0
              (parser
                 [: s = sig_item_0 :] ->
                   let _ =
                     try P.option (P.token ("", ";;")) strm__ with
                     [ Stream.Failure -> raise (Stream.Error "") ]
                   in
                   s)) :] ->
      sg
and mod_expr_ident_0 =
  parser bp
    [: :] ->
      let a = mod_expr_ident_1 strm__ in
      mod_expr_ident_gen_cont bp a strm__
and mod_expr_ident_0_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       j = P.orzero mod_expr_ident_1 mod_expr_ident_0
         ? P.error "mod_expr_ident" "'.'" "[mod_expr_ident]" :] ->
      let loc = P.gloc bp strm__ in
      let i = a__ in
      mod_expr_ident_0_cont bp (MLast.MeAcc loc i j) strm__
  | [: :] -> a__ ]
and mod_expr_ident_1 =
  parser bp
    [: i = P.vala (P.token ("UIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.MeUid loc i
and mod_expr_ident_gen_cont bp a__ strm__ =
  let x = mod_expr_ident_0_cont bp a__ strm__ in
  if x == a__ then x else mod_expr_ident_gen_cont bp x strm__
and constrain_0 =
  parser bp
    [: `("", "constraint");
       t1 = ctyp_0 ? P.error "constrain" "'constraint'" "[ctyp]";
       `("", "=") ? P.error "constrain" "[ctyp]" "'='";
       t2 = ctyp_0 ? P.error "constrain" "'='" "[ctyp]" :] ->
      (t1, t2)
and label_declaration_0 =
  parser bp
    [: :] ->
      match Stream.npeek 3 strm__ with
      [ [("", "mutable"); ("LIDENT", i); ("", ":")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          Stream.junk strm__;
          match
            try Some (poly_type_0 strm__) with [ Stream.Failure -> None ]
          with
          [ Some t ->
              let loc = P.gloc bp strm__ in
              (loc, i, True, t)
          | _ ->
              raise
                (Stream.Error
                   (P.error "label_declaration" "'mutable'" "[poly_type]")) ]
        }
      | _ ->
          match Stream.npeek 2 strm__ with
          [ [("LIDENT", i); ("", ":")] -> do {
              Stream.junk strm__;
              Stream.junk strm__;
              match
                try Some (poly_type_0 strm__) with [ Stream.Failure -> None ]
              with
              [ Some t ->
                  let loc = P.gloc bp strm__ in
                  (loc, i, False, t)
              | _ ->
                  raise
                    (Stream.Error
                       (P.error "label_declaration" "lowercase identifier"
                          "[poly_type]")) ]
            }
          | _ -> raise Stream.Failure ] ]
and label_declarations_0 =
  parser bp
    [: ld = label_declaration_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (label_declarations_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some ldl -> [ld :: ldl]
              | _ -> [ld] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "label_declarations" "';'"
                        "[label_declarations]")) ]
        | [: :] -> [ld] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "label_declarations" "[label_declaration]" "';'")) ]
and type_kind_0 =
  parser bp
  [ [: `("", "{");
       ldl = P.vala label_declarations_0
         ? P.error "type_kind" "'{'" "[label_declarations]";
       `("", "}") ? P.error "type_kind" "[label_declarations]" "'}'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyRec loc ldl
  | [: _ = test_constr_decl :] ->
      let _ =
        try P.option (P.token ("", "|")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match
        try
          Some
            (P.list1sep constructor_declaration_0 (P.token ("", "|")) False
               strm__)
        with
        [ Stream.Failure -> None ]
      with
      [ Some cdl ->
          let loc = P.gloc bp strm__ in
          MLast.TySum loc (Ploc.VaVal cdl)
      | _ ->
          raise
            (Stream.Error
               (P.error "type_kind" "" "[constructor_declaration]")) ]
  | [: t = ctyp_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "=") :] ->
            let pf =
              try P.bool (P.token ("", "private")) strm__ with
              [ Stream.Failure -> raise (Stream.Error "") ]
            in
            try
              match strm__ with parser
              [ [: `("", "{");
                   ldl = P.vala label_declarations_0
                     ? P.error "type_kind" "'{'" "[label_declarations]";
                   `("", "}")
                     ? P.error "type_kind" "[label_declarations]" "'}'" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.TyMan loc t (Ploc.VaVal pf) (MLast.TyRec loc ldl)
              | [: :] ->
                  let _ =
                    try P.option (P.token ("", "|")) strm__ with
                    [ Stream.Failure -> raise (Stream.Error "") ]
                  in
                  match
                    try
                      Some
                        (P.list1sep constructor_declaration_0
                           (P.token ("", "|")) False strm__)
                    with
                    [ Stream.Failure -> None ]
                  with
                  [ Some cdl ->
                      let loc = P.gloc bp strm__ in
                      MLast.TyMan loc t (Ploc.VaVal pf)
                        (MLast.TySum loc (Ploc.VaVal cdl))
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "type_kind" ""
                              "[constructor_declaration]")) ] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "type_kind" ""
                        "'{' or '|' or [constructor_declaration]")) ]
        | [: :] -> t ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "type_kind" "[ctyp]" "'='")) ] ]
and type_patt_0 =
  parser bp
    [: n = P.vala (P.token ("LIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      (loc, n)
and type_declaration_0 =
  parser bp
    [: tpl = type_parameters_0;
       n = P.vala type_patt_0
         ? P.error "type_declaration" "[type_parameters]" "[type_patt]" :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             pf = P.vala (P.bool (P.token ("", "private")))
               ? P.error "type_declaration" "'='" "???";
             tk = type_kind_0
               ? P.error "type_declaration" "???" "[type_kind]";
             cl = P.vala (P.list0 constrain_0)
               ? P.error "type_declaration" "[type_kind]" "???" :] ->
            let loc = P.gloc bp strm__ in
            {MLast.tdNam = n; MLast.tdPrm = Ploc.VaVal tpl; MLast.tdPrv = pf;
             MLast.tdDef = tk; MLast.tdCon = cl}
        | [: cl = P.vala (P.list0 constrain_0) :] ->
            let loc = P.gloc bp strm__ in
            let tk = MLast.TyQuo loc (Ploc.VaVal (choose_tvar tpl)) in
            {MLast.tdNam = n; MLast.tdPrm = Ploc.VaVal tpl;
             MLast.tdPrv = Ploc.VaVal False; MLast.tdDef = tk;
             MLast.tdCon = cl} ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "type_declaration" "[type_patt]" "'=' or ???")) ]
and mod_fun_binding_0 =
  parser bp
  [ [: `("", ":");
       mt = module_type_0 ? P.error "mod_fun_binding" "':'" "[module_type]";
       `("", "=") ? P.error "mod_fun_binding" "[module_type]" "'='";
       me = module_expr_0
         ? P.error "mod_fun_binding" "'='" "[module_expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MeTyc loc me mt
  | [: `("", "=");
       me = module_expr_0
         ? P.error "mod_fun_binding" "'='" "[module_expr]" :] ->
      me
  | [: arg = P.vala functor_parameter_0;
       mb = mod_fun_binding_0
         ? P.error "mod_fun_binding" "[functor_parameter]"
             "[mod_fun_binding]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MeFun loc arg mb ]
and mod_binding_0 =
  parser bp
    [: i = P.vala uidopt_0;
       me = mod_fun_binding_0
         ? P.error "mod_binding" "[uidopt]" "[mod_fun_binding]" :] ->
      (i, me)
and rebind_exn_0 =
  parser bp
  [ [: `("", "=");
       sl = P.vala mod_ident_0
         ? P.error "rebind_exn" "'='" "[mod_ident]" :] ->
      sl
  | [: :] ->
      let loc = P.gloc bp strm__ in
      Ploc.VaVal [] ]
and cons_ident_0 =
  parser bp
  [ [: `("UIDENT", "True") :] ->
      let loc = P.gloc bp strm__ in
      Ploc.VaVal "True_"
  | [: `("UIDENT", "False") :] ->
      let loc = P.gloc bp strm__ in
      Ploc.VaVal "False_"
  | [: i = P.vala (P.token ("UIDENT", "")) :] -> i ]
and constructor_declaration_0 =
  parser bp
    [: ci = cons_ident_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "of");
             cal = P.vala (P.list1sep ctyp_4 (P.token ("", "*")) False)
               ? P.error "constructor_declaration" "'of'" "???" :] ->
            let loc = P.gloc bp strm__ in
            (loc, ci, cal, None)
        | [: `("", ":");
             cal = P.vala (P.list1sep ctyp_4 (P.token ("", "*")) False)
               ? P.error "constructor_declaration" "':'" "???" :] ->
            try
              match strm__ with parser
              [ [: `("", "->");
                   t = ctyp_0
                     ? P.error "constructor_declaration" "'->'" "[ctyp]" :] ->
                  let loc = P.gloc bp strm__ in
                  (loc, ci, cal, Some t)
              | [: :] ->
                  let loc = P.gloc bp strm__ in
                  let t =
                    match cal with
                    [ Ploc.VaVal [t] -> t
                    | Ploc.VaVal [t :: tl] ->
                        MLast.TyTup loc (Ploc.VaVal [t :: tl])
                    | _ -> assert False ]
                  in
                  (loc, ci, Ploc.VaVal [], Some t) ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "constructor_declaration" "???" "'->'")) ]
        | [: :] ->
            let loc = P.gloc bp strm__ in
            (loc, ci, Ploc.VaVal [], None) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "constructor_declaration" "[cons_ident]"
                  "'of' or ':'")) ]
and class_type_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("LIDENT", i); ("", ":")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          match try Some (ctyp_4 strm__) with [ Stream.Failure -> None ] with
          [ Some t ->
              match strm__ with parser
              [ [: `("", "->");
                   ct = class_type_0
                     ? P.error "class_type" "'->'" "[class_type]" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.CtFun loc (MLast.TyLab loc (Ploc.VaVal i) t) ct
              | [: :] ->
                  raise
                    (Stream.Error
                       (P.error "class_type" "[ctyp level apply]" "'->'")) ]
          | _ ->
              raise
                (Stream.Error
                   (P.error "class_type" "lowercase identifier"
                      "[ctyp level apply]")) ]
        }
      | _ ->
          match
            try Some (P.vala (P.token ("QUESTIONIDENTCOLON", "")) strm__) with
            [ Stream.Failure -> None ]
          with
          [ Some i ->
              match
                try Some (ctyp_4 strm__) with [ Stream.Failure -> None ]
              with
              [ Some t ->
                  match strm__ with parser
                  [ [: `("", "->");
                       ct = class_type_0
                         ? P.error "class_type" "'->'" "[class_type]" :] ->
                      let loc = P.gloc bp strm__ in
                      MLast.CtFun loc (MLast.TyOlb loc i t) ct
                  | [: :] ->
                      raise
                        (Stream.Error
                           (P.error "class_type" "[ctyp level apply]"
                              "'->'")) ]
              | _ ->
                  raise
                    (Stream.Error
                       (P.error "class_type" "QUESTIONIDENTCOLON"
                          "[ctyp level apply]")) ]
          | _ ->
              match Stream.npeek 2 strm__ with
              [ [("QUESTIONIDENT", i); ("", ":")] -> do {
                  Stream.junk strm__;
                  Stream.junk strm__;
                  let i = Ploc.VaVal i in
                  match
                    try Some (ctyp_4 strm__) with [ Stream.Failure -> None ]
                  with
                  [ Some t ->
                      match strm__ with parser
                      [ [: `("", "->");
                           ct = class_type_0
                             ? P.error "class_type" "'->'"
                                 "[class_type]" :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.CtFun loc (MLast.TyOlb loc i t) ct
                      | [: :] ->
                          raise
                            (Stream.Error
                               (P.error "class_type" "[ctyp level apply]"
                                  "'->'")) ]
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "class_type" "QUESTIONIDENT"
                              "[ctyp level apply]")) ]
                }
              | _ ->
                  match
                    try Some (test_ctyp_minusgreater strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some _ ->
                      match
                        try Some (ctyp_3 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some t ->
                          match strm__ with parser
                          [ [: `("", "->");
                               ct = class_type_0
                                 ? P.error "class_type" "'->'"
                                     "[class_type]" :] ->
                              let loc = P.gloc bp strm__ in
                              MLast.CtFun loc t ct
                          | [: :] ->
                              raise
                                (Stream.Error
                                   (P.error "class_type" "[ctyp level star]"
                                      "'->'")) ]
                      | _ ->
                          raise
                            (Stream.Error
                               (P.error "class_type"
                                  "[test_ctyp_minusgreater]"
                                  "[ctyp level star]")) ]
                  | _ ->
                      match
                        try Some (class_signature_0 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some cs -> cs
                      | _ -> raise Stream.Failure ] ] ] ] ]
and cvalue_binding_0 =
  parser bp
  [ [: `("", "="); e = expr_0 ? P.error "cvalue_binding" "'='" "[expr]" :] ->
      e
  | [: `("", ":"); t = ctyp_0 ? P.error "cvalue_binding" "':'" "[ctyp]" :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             e = expr_0 ? P.error "cvalue_binding" "'='" "[expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExTyc loc e t
        | [: `("", ":>");
             t2 = ctyp_0 ? P.error "cvalue_binding" "':>'" "[ctyp]";
             `("", "=") ? P.error "cvalue_binding" "[ctyp]" "'='";
             e = expr_0 ? P.error "cvalue_binding" "'='" "[expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExCoe loc e (Some t) t2 ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "cvalue_binding" "[ctyp]" "'=' or ':>'")) ]
  | [: `("", ":>"); t = ctyp_0 ? P.error "cvalue_binding" "':>'" "[ctyp]";
       `("", "=") ? P.error "cvalue_binding" "[ctyp]" "'='";
       e = expr_0 ? P.error "cvalue_binding" "'='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExCoe loc e None t ]
and class_str_item_0 =
  parser bp
  [ [: `("", "inherit");
       ce = class_expr_0
         ? P.error "class_str_item" "'inherit'" "[class_expr]";
       pb =
         P.vala
           (P.option
              (parser
                 [: :] ->
                   match Stream.npeek 2 strm__ with
                   [ [("", "as"); ("LIDENT", i)] -> do {
                       Stream.junk strm__;
                       Stream.junk strm__;
                       try i with
                       [ Stream.Failure ->
                           raise
                             (Stream.Error (P.error "implem" "'as'" "???")) ]
                     }
                   | _ -> raise Stream.Failure ]))
         ? P.error "class_str_item" "[class_expr]" "???" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CrInh loc ce pb
  | [: `("", "val") :] ->
      try
        match strm__ with parser
        [ [: `("", "virtual");
             mf = P.vala (P.bool (P.token ("", "mutable")))
               ? P.error "class_str_item" "'virtual'" "???" :] ->
            match Stream.npeek 2 strm__ with
            [ [("LIDENT", lab); ("", ":")] -> do {
                Stream.junk strm__;
                Stream.junk strm__;
                let lab = Ploc.VaVal lab in
                match
                  try Some (ctyp_0 strm__) with [ Stream.Failure -> None ]
                with
                [ Some t ->
                    let loc = P.gloc bp strm__ in
                    MLast.CrVav loc mf lab t
                | _ ->
                    raise
                      (Stream.Error
                         (P.error "class_str_item" "lowercase identifier"
                            "[ctyp]")) ]
              }
            | _ ->
                raise
                  (Stream.Error
                     (P.error "class_str_item" "???"
                        "lowercase identifier")) ]
        | [: ov = P.vala (P.bool (P.token ("", "!")));
             mf = P.vala (P.bool (P.token ("", "mutable")))
               ? P.error "class_str_item" "???" "???" :] ->
            try
              match Stream.npeek 3 strm__ with
              [ [("", "virtual"); ("LIDENT", lab); ("", ":")] -> do {
                  Stream.junk strm__;
                  Stream.junk strm__;
                  Stream.junk strm__;
                  let lab = Ploc.VaVal lab in
                  match
                    try Some (ctyp_0 strm__) with [ Stream.Failure -> None ]
                  with
                  [ Some t ->
                      let loc = P.gloc bp strm__ in
                      if Pcaml.unvala ov then
                        Ploc.raise loc
                          (Stream.Error "virtual value cannot override")
                      else MLast.CrVav loc mf lab t
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "class_str_item" "'virtual'" "[ctyp]")) ]
                }
              | _ ->
                  match
                    try Some (P.vala (P.token ("LIDENT", "")) strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some lab ->
                      match
                        try Some (cvalue_binding_0 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some e ->
                          let loc = P.gloc bp strm__ in
                          MLast.CrVal loc ov mf lab e
                      | _ ->
                          raise
                            (Stream.Error
                               (P.error "class_str_item"
                                  "lowercase identifier"
                                  "[cvalue_binding]")) ]
                  | _ -> raise Stream.Failure ] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "class_str_item" "???"
                        "'virtual' or lowercase identifier")) ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "class_str_item" "'val'" "'virtual' or ???")) ]
  | [: `("", "method") :] ->
      try
        match Stream.npeek 4 strm__ with
        [ [("", "private"); ("", "virtual"); ("LIDENT", l); ("", ":")] -> do {
            Stream.junk strm__;
            Stream.junk strm__;
            Stream.junk strm__;
            Stream.junk strm__;
            let l = Ploc.VaVal l in
            match
              try Some (poly_type_0 strm__) with [ Stream.Failure -> None ]
            with
            [ Some t ->
                let loc = P.gloc bp strm__ in
                MLast.CrVir loc (Ploc.VaVal True) l t
            | _ ->
                raise
                  (Stream.Error
                     (P.error "class_str_item" "'private'" "[poly_type]")) ]
          }
        | _ ->
            match strm__ with parser
            [ [: `("", "virtual") :] ->
                try
                  match Stream.npeek 3 strm__ with
                  [ [("", "private"); ("LIDENT", l); ("", ":")] -> do {
                      Stream.junk strm__;
                      Stream.junk strm__;
                      Stream.junk strm__;
                      let l = Ploc.VaVal l in
                      match
                        try Some (poly_type_0 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some t ->
                          let loc = P.gloc bp strm__ in
                          MLast.CrVir loc (Ploc.VaVal True) l t
                      | _ ->
                          raise
                            (Stream.Error
                               (P.error "class_str_item" "'private'"
                                  "[poly_type]")) ]
                    }
                  | _ ->
                      match Stream.npeek 2 strm__ with
                      [ [("LIDENT", l); ("", ":")] -> do {
                          Stream.junk strm__;
                          Stream.junk strm__;
                          let l = Ploc.VaVal l in
                          match
                            try Some (poly_type_0 strm__) with
                            [ Stream.Failure -> None ]
                          with
                          [ Some t ->
                              let loc = P.gloc bp strm__ in
                              MLast.CrVir loc (Ploc.VaVal False) l t
                          | _ ->
                              raise
                                (Stream.Error
                                   (P.error "class_str_item"
                                      "lowercase identifier" "[poly_type]")) ]
                        }
                      | _ -> raise Stream.Failure ] ]
                with
                [ Stream.Failure ->
                    raise
                      (Stream.Error
                         (P.error "class_str_item" "'virtual'"
                            "'private' or lowercase identifier")) ]
            | [: ov = P.vala (P.bool (P.token ("", "!"))) :] ->
                try
                  match Stream.npeek 2 strm__ with
                  [ [("", "private"); ("LIDENT", l)] -> do {
                      Stream.junk strm__;
                      Stream.junk strm__;
                      let l = Ploc.VaVal l in
                      try
                        match strm__ with parser
                        [ [: `("", ":");
                             t = poly_type_0
                               ? P.error "class_str_item" "':'" "[poly_type]";
                             `("", "=")
                               ? P.error "class_str_item" "[poly_type]" "'='";
                             e = expr_0
                               ? P.error "class_str_item" "'='" "[expr]" :] ->
                            let loc = P.gloc bp strm__ in
                            MLast.CrMth loc ov (Ploc.VaVal True) l
                              (Ploc.VaVal (Some t)) e
                        | [: sb = fun_binding_0 :] ->
                            let loc = P.gloc bp strm__ in
                            MLast.CrMth loc ov (Ploc.VaVal True) l
                              (Ploc.VaVal None) sb ]
                      with
                      [ Stream.Failure ->
                          raise
                            (Stream.Error
                               (P.error "class_str_item" "'private'"
                                  "':' or [fun_binding]")) ]
                    }
                  | _ ->
                      match
                        try Some (P.vala (P.token ("LIDENT", "")) strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some l ->
                          try
                            match strm__ with parser
                            [ [: `("", ":");
                                 t = poly_type_0
                                   ? P.error "class_str_item" "':'"
                                       "[poly_type]";
                                 `("", "=")
                                   ? P.error "class_str_item" "[poly_type]"
                                       "'='";
                                 e = expr_0
                                   ? P.error "class_str_item" "'='"
                                       "[expr]" :] ->
                                let loc = P.gloc bp strm__ in
                                MLast.CrMth loc ov (Ploc.VaVal False) l
                                  (Ploc.VaVal (Some t)) e
                            | [: sb = fun_binding_0 :] ->
                                let loc = P.gloc bp strm__ in
                                MLast.CrMth loc ov (Ploc.VaVal False) l
                                  (Ploc.VaVal None) sb ]
                          with
                          [ Stream.Failure ->
                              raise
                                (Stream.Error
                                   (P.error "class_str_item"
                                      "lowercase identifier"
                                      "':' or [fun_binding]")) ]
                      | _ -> raise Stream.Failure ] ]
                with
                [ Stream.Failure ->
                    raise
                      (Stream.Error
                         (P.error "class_str_item" "???"
                            "'private' or lowercase identifier")) ] ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "class_str_item" "'method'"
                  "'private' or 'virtual' or ???")) ]
  | [: `("", "constraint");
       t1 = ctyp_0 ? P.error "class_str_item" "'constraint'" "[ctyp]";
       `("", "=") ? P.error "class_str_item" "[ctyp]" "'='";
       t2 = ctyp_0 ? P.error "class_str_item" "'='" "[ctyp]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CrCtr loc t1 t2
  | [: `("", "initializer");
       se = expr_0 ? P.error "class_str_item" "'initializer'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CrIni loc se ]
and class_structure_0 =
  parser bp
    [: :] ->
      let cf =
        try P.list0 class_str_item_0 strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      cf
and class_self_patt_0 =
  parser bp
    [: `("", "("); p = patt_0 ? P.error "class_self_patt" "'('" "[patt]" :] ->
      try
        match strm__ with parser
        [ [: `("", ")") :] -> p
        | [: `("", ":");
             t = ctyp_0 ? P.error "class_self_patt" "':'" "[ctyp]";
             `("", ")") ? P.error "class_self_patt" "[ctyp]" "')'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaTyc loc p t ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "class_self_patt" "[patt]" "')' or ':'")) ]
and fun_binding_0 =
  parser bp
  [ [: `("", "="); e = expr_0 ? P.error "fun_binding" "'='" "[expr]" :] -> e
  | [: `("", ":");
       t = poly_type_0 ? P.error "fun_binding" "':'" "[poly_type]";
       `("", "=") ? P.error "fun_binding" "[poly_type]" "'='";
       e = expr_0 ? P.error "fun_binding" "'='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExTyc loc e t
  | [: p = labeled_patt_0;
       e = fun_binding_0
         ? P.error "fun_binding" "[labeled_patt]" "[fun_binding]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExFun loc (Ploc.VaVal [(p, Ploc.VaVal None, e)])
  | [: p = patt_7;
       e = fun_binding_0
         ? P.error "fun_binding" "[patt level simple]" "[fun_binding]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExFun loc (Ploc.VaVal [(p, Ploc.VaVal None, e)]) ]
and val_ident_0 =
  parser bp
    [: _ = check_not_part_of_patt :] ->
      try
        match Stream.npeek 3 strm__ with
        [ [("", "("); (_, s); ("", ")")] -> do {
            Stream.junk strm__;
            Stream.junk strm__;
            Stream.junk strm__;
            try
              let loc = P.gloc bp strm__ in
              MLast.PaLid loc (Ploc.VaVal s)
            with
            [ Stream.Failure ->
                raise (Stream.Error (P.error "val_ident" "'('" "???")) ]
          }
        | _ ->
            match strm__ with parser
              [: `("LIDENT", s) :] ->
                let loc = P.gloc bp strm__ in
                MLast.PaLid loc (Ploc.VaVal s) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "val_ident" "[check_not_part_of_patt]"
                  "'(' or lowercase identifier")) ]
and let_binding_0 =
  parser bp
  [ [: p = val_ident_0;
       e = fun_binding_0
         ? P.error "let_binding" "[val_ident]" "[fun_binding]" :] ->
      (p, e)
  | [: p = patt_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             e = expr_0 ? P.error "let_binding" "'='" "[expr]" :] ->
            (p, e)
        | [: `("", ":");
             t = poly_type_0 ? P.error "let_binding" "':'" "[poly_type]";
             `("", "=") ? P.error "let_binding" "[poly_type]" "'='";
             e = expr_0 ? P.error "let_binding" "'='" "[expr]" :] ->
            let loc = P.gloc bp strm__ in
            (MLast.PaTyc loc p t, e) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "let_binding" "[patt]" "'=' or ':'")) ] ]
and labeled_patt_0 =
  parser bp
    [: :] ->
      match Stream.npeek 3 strm__ with
      [ [("", "~"); ("", "("); ("LIDENT", i)] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          Stream.junk strm__;
          try
            match strm__ with parser
            [ [: `("", ")") :] ->
                let loc = P.gloc bp strm__ in
                MLast.PaLab loc
                  (Ploc.VaVal
                     [(MLast.PaLid loc (Ploc.VaVal i), Ploc.VaVal None)])
            | [: `("", ":");
                 t = ctyp_0 ? P.error "labeled_patt" "':'" "[ctyp]";
                 `("", ")") ? P.error "labeled_patt" "[ctyp]" "')'" :] ->
                let loc = P.gloc bp strm__ in
                MLast.PaLab loc
                  (Ploc.VaVal
                     [(MLast.PaTyc loc (MLast.PaLid loc (Ploc.VaVal i)) t,
                       Ploc.VaVal None)]) ]
          with
          [ Stream.Failure ->
              raise
                (Stream.Error (P.error "labeled_patt" "'~'" "')' or ':'")) ]
        }
      | _ ->
          match Stream.npeek 3 strm__ with
          [ [("", "?"); ("", "("); ("LIDENT", i)] -> do {
              Stream.junk strm__;
              Stream.junk strm__;
              Stream.junk strm__;
              try
                match strm__ with parser
                [ [: `("", "=");
                     e = expr_0 ? P.error "labeled_patt" "'='" "[expr]";
                     `("", ")") ? P.error "labeled_patt" "[expr]" "')'" :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.PaOlb loc (MLast.PaLid loc (Ploc.VaVal i))
                      (Ploc.VaVal (Some e))
                | [: `("", ")") :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.PaOlb loc (MLast.PaLid loc (Ploc.VaVal i))
                      (Ploc.VaVal None)
                | [: `("", ":");
                     t = ctyp_0 ? P.error "labeled_patt" "':'" "[ctyp]" :] ->
                    try
                      match strm__ with parser
                      [ [: `("", "=");
                           e = expr_0 ? P.error "labeled_patt" "'='" "[expr]";
                           `("", ")")
                             ? P.error "labeled_patt" "[expr]" "')'" :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.PaOlb loc
                            (MLast.PaTyc loc (MLast.PaLid loc (Ploc.VaVal i))
                               t)
                            (Ploc.VaVal (Some e))
                      | [: `("", ")") :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.PaOlb loc
                            (MLast.PaTyc loc (MLast.PaLid loc (Ploc.VaVal i))
                               t)
                            (Ploc.VaVal None) ]
                    with
                    [ Stream.Failure ->
                        raise
                          (Stream.Error
                             (P.error "labeled_patt" "[ctyp]"
                                "'=' or ')'")) ] ]
              with
              [ Stream.Failure ->
                  raise
                    (Stream.Error
                       (P.error "labeled_patt" "'?'" "'=' or ')' or ':'")) ]
            }
          | _ ->
              match
                try
                  Some (P.vala (P.token ("TILDEIDENTCOLON", "")) strm__)
                with
                [ Stream.Failure -> None ]
              with
              [ Some i ->
                  match
                    try Some (patt_7 strm__) with [ Stream.Failure -> None ]
                  with
                  [ Some p ->
                      let loc = P.gloc bp strm__ in
                      MLast.PaLab loc
                        (Ploc.VaVal
                           [(MLast.PaLid loc i, Ploc.VaVal (Some p))])
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "labeled_patt" "TILDEIDENTCOLON"
                              "[patt level simple]")) ]
              | _ ->
                  match
                    try Some (P.vala (P.token ("TILDEIDENT", "")) strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some i ->
                      let loc = P.gloc bp strm__ in
                      MLast.PaLab loc
                        (Ploc.VaVal [(MLast.PaLid loc i, Ploc.VaVal None)])
                  | _ ->
                      match
                        try
                          Some
                            (P.vala (P.token ("QUESTIONIDENTCOLON", ""))
                               strm__)
                        with
                        [ Stream.Failure -> None ]
                      with
                      [ Some i ->
                          try
                            match strm__ with parser
                            [ [: `("", "_") :] ->
                                let loc = P.gloc bp strm__ in
                                MLast.PaOlb loc (MLast.PaLid loc i)
                                  (Ploc.VaVal None)
                            | [: `("", "(");
                                 p = patt_0
                                   ? P.error "labeled_patt" "'('"
                                       "[patt]" :] ->
                                try
                                  match strm__ with parser
                                  [ [: `("", "=");
                                       e = expr_0
                                         ? P.error "labeled_patt" "'='"
                                             "[expr]";
                                       `("", ")")
                                         ? P.error "labeled_patt" "[expr]"
                                             "')'" :] ->
                                      let loc = P.gloc bp strm__ in
                                      MLast.PaOlb loc (MLast.PaLid loc i)
                                        (Ploc.VaVal
                                           (Some
                                              (MLast.ExOlb loc p
                                                 (Ploc.VaVal (Some e)))))
                                  | [: `("", ":");
                                       t = ctyp_0
                                         ? P.error "labeled_patt" "':'"
                                             "[ctyp]" :] ->
                                      try
                                        match strm__ with parser
                                        [ [: `("", ")") :] ->
                                            let loc = P.gloc bp strm__ in
                                            MLast.PaOlb loc
                                              (MLast.PaLid loc i)
                                              (Ploc.VaVal
                                                 (Some
                                                    (MLast.ExOlb loc
                                                       (MLast.PaTyc loc p t)
                                                       (Ploc.VaVal None))))
                                        | [: `("", "=");
                                             e = expr_0
                                               ? P.error "labeled_patt" "'='"
                                                   "[expr]";
                                             `("", ")")
                                               ? P.error "labeled_patt"
                                                   "[expr]" "')'" :] ->
                                            let loc = P.gloc bp strm__ in
                                            MLast.PaOlb loc
                                              (MLast.PaLid loc i)
                                              (Ploc.VaVal
                                                 (Some
                                                    (MLast.ExOlb loc
                                                       (MLast.PaTyc loc p t)
                                                       (Ploc.VaVal
                                                          (Some e))))) ]
                                      with
                                      [ Stream.Failure ->
                                          raise
                                            (Stream.Error
                                               (P.error "labeled_patt"
                                                  "[ctyp]" "')' or '='")) ]
                                  | [: `("", ")") :] ->
                                      let loc = P.gloc bp strm__ in
                                      MLast.PaOlb loc (MLast.PaLid loc i)
                                        (Ploc.VaVal
                                           (Some
                                              (MLast.ExOlb loc p
                                                 (Ploc.VaVal None)))) ]
                                with
                                [ Stream.Failure ->
                                    raise
                                      (Stream.Error
                                         (P.error "labeled_patt" "[patt]"
                                            "'=' or ':' or ')'")) ]
                            | [: `("LIDENT", j) :] ->
                                let loc = P.gloc bp strm__ in
                                MLast.PaOlb loc (MLast.PaLid loc i)
                                  (Ploc.VaVal
                                     (Some
                                        (MLast.ExOlb loc
                                           (MLast.PaLid loc (Ploc.VaVal j))
                                           (Ploc.VaVal None)))) ]
                          with
                          [ Stream.Failure ->
                              raise
                                (Stream.Error
                                   (P.error "labeled_patt"
                                      "QUESTIONIDENTCOLON"
                                      "'_' or '(' or lowercase identifier")) ]
                      | _ ->
                          match
                            try
                              Some
                                (P.vala (P.token ("QUESTIONIDENT", ""))
                                   strm__)
                            with
                            [ Stream.Failure -> None ]
                          with
                          [ Some i ->
                              let loc = P.gloc bp strm__ in
                              MLast.PaOlb loc (MLast.PaLid loc i)
                                (Ploc.VaVal None)
                          | _ -> raise Stream.Failure ] ] ] ] ] ]
and class_fun_def_0 =
  parser bp
  [ [: p = labeled_patt_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "->");
             ce = class_expr_0
               ? P.error "class_fun_def" "'->'" "[class_expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeFun loc p ce
        | [: cfd = class_fun_def_0 :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeFun loc p cfd ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "class_fun_def" "[labeled_patt]"
                  "'->' or [class_fun_def]")) ]
  | [: p = patt_7 :] ->
      try
        match strm__ with parser
        [ [: `("", "->");
             ce = class_expr_0
               ? P.error "class_fun_def" "'->'" "[class_expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeFun loc p ce
        | [: cfd = class_fun_def_0 :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeFun loc p cfd ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "class_fun_def" "[patt level simple]"
                  "'->' or [class_fun_def]")) ] ]
and class_expr_0 =
  parser bp
  [ [: `("", "fun");
       cfd = class_fun_def_0
         ? P.error "class_expr" "'fun'" "[class_fun_def]" :] ->
      cfd
  | [: `("", "let");
       rf = P.vala (P.bool (P.token ("", "rec")))
         ? P.error "class_expr" "'let'" "???";
       lb = P.vala (P.list1sep let_binding_0 (P.token ("", "and")) False)
         ? P.error "class_expr" "???" "???";
       `("", "in") ? P.error "class_expr" "???" "'in'";
       ce = class_expr_0 ? P.error "class_expr" "'in'" "[class_expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CeLet loc rf lb ce
  | [: :] -> class_expr_1 strm__ ]
and class_expr_1 =
  parser bp
    [: :] ->
      let a = class_expr_2 strm__ in
      class_expr_1_cont bp a strm__
and class_expr_1_cont bp a__ strm__ =
  match try Some (expr_14 strm__) with [ Stream.Failure -> None ] with
  [ Some e ->
      let loc = P.gloc bp strm__ in
      let ce = a__ in
      class_expr_1_cont bp (MLast.CeApp loc ce e) strm__
  | _ -> a__ ]
and class_expr_2 =
  parser bp
  [ [: `("", "["); ct = ctyp_0 ? P.error "class_expr" "'['" "[ctyp]" :] ->
      try
        match strm__ with parser
        [ [: `("", ",");
             ctcl = P.list1sep ctyp_0 (P.token ("", ",")) False
               ? P.error "class_expr" "','" "[ctyp]";
             `("", "]") ? P.error "class_expr" "" "[ctyp] or ']'";
             ci = class_longident_0
               ? P.error "class_expr" "']'" "[class_longident]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeCon loc (Ploc.VaVal ci) (Ploc.VaVal [ct :: ctcl])
        | [: `("", "]");
             ci = class_longident_0
               ? P.error "class_expr" "']'" "[class_longident]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeCon loc (Ploc.VaVal ci) (Ploc.VaVal [ct]) ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "class_expr" "[ctyp]" "',' or ']'")) ]
  | [: `("", "object");
       cspo = P.vala (P.option class_self_patt_0)
         ? P.error "class_expr" "'object'" "???";
       cf = P.vala class_structure_0
         ? P.error "class_expr" "???" "[class_structure]";
       `("", "end") ? P.error "class_expr" "[class_structure]" "'end'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CeStr loc cspo cf
  | [: `("", "(");
       ce = class_expr_0 ? P.error "class_expr" "'('" "[class_expr]" :] ->
      try
        match strm__ with parser
        [ [: `("", ":");
             ct = class_type_0 ? P.error "class_expr" "':'" "[class_type]";
             `("", ")") ? P.error "class_expr" "[class_type]" "')'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.CeTyc loc ce ct
        | [: `("", ")") :] -> ce ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "class_expr" "[class_expr]" "':' or ')'")) ]
  | [: ci = class_longident_0 :] ->
      let loc = P.gloc bp strm__ in
      MLast.CeCon loc (Ploc.VaVal ci) (Ploc.VaVal []) ]
and class_expr_gen_cont bp a__ strm__ =
  let x = class_expr_1_cont bp a__ strm__ in
  if x == a__ then x else class_expr_gen_cont bp x strm__
and class_fun_binding_0 =
  parser bp
  [ [: `("", "=");
       ce = class_expr_0
         ? P.error "class_fun_binding" "'='" "[class_expr]" :] ->
      ce
  | [: `("", ":");
       ct = class_type_0 ? P.error "class_fun_binding" "':'" "[class_type]";
       `("", "=") ? P.error "class_fun_binding" "[class_type]" "'='";
       ce = class_expr_0
         ? P.error "class_fun_binding" "'='" "[class_expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CeTyc loc ce ct
  | [: p = labeled_patt_0;
       cfb = class_fun_binding_0
         ? P.error "class_fun_binding" "[labeled_patt]"
             "[class_fun_binding]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CeFun loc p cfb
  | [: p = patt_7;
       cfb = class_fun_binding_0
         ? P.error "class_fun_binding" "[patt level simple]"
             "[class_fun_binding]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CeFun loc p cfb ]
and class_declaration_0 =
  parser bp
    [: vf = P.vala (P.bool (P.token ("", "virtual")));
       ctp = class_type_parameters_0
         ? P.error "class_declaration" "???" "[class_type_parameters]";
       i = P.vala (P.token ("LIDENT", ""))
         ? P.error "class_declaration" "[class_type_parameters]"
             "lowercase identifier";
       cfb = class_fun_binding_0
         ? P.error "class_declaration" "lowercase identifier"
             "[class_fun_binding]" :] ->
      let loc = P.gloc bp strm__ in
      {MLast.ciLoc = loc; MLast.ciVir = vf; MLast.ciPrm = ctp;
       MLast.ciNam = i; MLast.ciExp = cfb}
and class_sig_item_0 =
  parser bp
  [ [: `("", "inherit");
       cs = class_signature_0
         ? P.error "class_sig_item" "'inherit'" "[class_signature]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CgInh loc cs
  | [: `("", "val");
       mf = P.vala (P.bool (P.token ("", "mutable")))
         ? P.error "class_sig_item" "'val'" "???" :] ->
      match Stream.npeek 2 strm__ with
      [ [("LIDENT", l); ("", ":")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let l = Ploc.VaVal l in
          match try Some (ctyp_0 strm__) with [ Stream.Failure -> None ] with
          [ Some t ->
              let loc = P.gloc bp strm__ in
              MLast.CgVal loc mf l t
          | _ ->
              raise
                (Stream.Error
                   (P.error "class_sig_item" "lowercase identifier"
                      "[ctyp]")) ]
        }
      | _ ->
          raise
            (Stream.Error
               (P.error "class_sig_item" "???" "lowercase identifier")) ]
  | [: `("", "method") :] ->
      try
        match strm__ with parser
        [ [: `("", "virtual") :] ->
            try
              match Stream.npeek 3 strm__ with
              [ [("", "private"); ("LIDENT", l); ("", ":")] -> do {
                  Stream.junk strm__;
                  Stream.junk strm__;
                  Stream.junk strm__;
                  let l = Ploc.VaVal l in
                  match
                    try Some (poly_type_0 strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some t ->
                      let loc = P.gloc bp strm__ in
                      MLast.CgVir loc (Ploc.VaVal True) l t
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "class_sig_item" "'private'"
                              "[poly_type]")) ]
                }
              | _ ->
                  match Stream.npeek 2 strm__ with
                  [ [("LIDENT", l); ("", ":")] -> do {
                      Stream.junk strm__;
                      Stream.junk strm__;
                      let l = Ploc.VaVal l in
                      match
                        try Some (poly_type_0 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some t ->
                          let loc = P.gloc bp strm__ in
                          MLast.CgVir loc (Ploc.VaVal False) l t
                      | _ ->
                          raise
                            (Stream.Error
                               (P.error "class_sig_item"
                                  "lowercase identifier" "[poly_type]")) ]
                    }
                  | _ -> raise Stream.Failure ] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "class_sig_item" "'virtual'"
                        "'private' or lowercase identifier")) ]
        | [: `("", "private") :] ->
            try
              match Stream.npeek 3 strm__ with
              [ [("", "virtual"); ("LIDENT", l); ("", ":")] -> do {
                  Stream.junk strm__;
                  Stream.junk strm__;
                  Stream.junk strm__;
                  let l = Ploc.VaVal l in
                  match
                    try Some (poly_type_0 strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some t ->
                      let loc = P.gloc bp strm__ in
                      MLast.CgVir loc (Ploc.VaVal True) l t
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "class_sig_item" "'virtual'"
                              "[poly_type]")) ]
                }
              | _ ->
                  match Stream.npeek 2 strm__ with
                  [ [("LIDENT", l); ("", ":")] -> do {
                      Stream.junk strm__;
                      Stream.junk strm__;
                      let l = Ploc.VaVal l in
                      match
                        try Some (poly_type_0 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some t ->
                          let loc = P.gloc bp strm__ in
                          MLast.CgMth loc (Ploc.VaVal True) l t
                      | _ ->
                          raise
                            (Stream.Error
                               (P.error "class_sig_item"
                                  "lowercase identifier" "[poly_type]")) ]
                    }
                  | _ -> raise Stream.Failure ] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "class_sig_item" "'private'"
                        "'virtual' or lowercase identifier")) ]
        | [: :] ->
            match Stream.npeek 2 strm__ with
            [ [("LIDENT", l); ("", ":")] -> do {
                Stream.junk strm__;
                Stream.junk strm__;
                let l = Ploc.VaVal l in
                match
                  try Some (poly_type_0 strm__) with
                  [ Stream.Failure -> None ]
                with
                [ Some t ->
                    let loc = P.gloc bp strm__ in
                    MLast.CgMth loc (Ploc.VaVal False) l t
                | _ ->
                    raise
                      (Stream.Error
                         (P.error "class_sig_item" "lowercase identifier"
                            "[poly_type]")) ]
              }
            | _ -> raise Stream.Failure ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "class_sig_item" "'method'"
                  "'virtual' or 'private' or lowercase identifier")) ]
  | [: `("", "constraint");
       t1 = ctyp_0 ? P.error "class_sig_item" "'constraint'" "[ctyp]";
       `("", "=") ? P.error "class_sig_item" "[ctyp]" "'='";
       t2 = ctyp_0 ? P.error "class_sig_item" "'='" "[ctyp]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CgCtr loc t1 t2 ]
and class_self_type_0 =
  parser bp
    [: `("", "("); t = ctyp_0 ? P.error "class_self_type" "'('" "[ctyp]";
       `("", ")") ? P.error "class_self_type" "[ctyp]" "')'" :] ->
      t
and class_signature_0 =
  parser bp
  [ [: `("", "[");
       tl = P.list1sep ctyp_0 (P.token ("", ",")) False
         ? P.error "class_signature" "'['" "[ctyp]";
       `("", "]") ? P.error "class_signature" "" "[ctyp] or ']'";
       id = class_signature_0
         ? P.error "class_signature" "']'" "[class_signature]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CtCon loc id (Ploc.VaVal tl)
  | [: `("", "object");
       cst = P.vala (P.option class_self_type_0)
         ? P.error "class_signature" "'object'" "???";
       csf = P.vala (P.list0 class_sig_item_0)
         ? P.error "class_signature" "???" "???";
       `("", "end") ? P.error "class_signature" "???" "'end'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.CtSig loc cst csf
  | [: :] -> class_signature_1 strm__ ]
and class_signature_1 =
  parser bp
    [: :] ->
      let a = class_signature_2 strm__ in
      class_signature_1_cont bp a strm__
and class_signature_1_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       ct2 = P.orzero class_signature_2 class_signature_0
         ? P.error "class_signature" "'.'" "[class_signature]" :] ->
      let loc = P.gloc bp strm__ in
      let ct1 = a__ in
      class_signature_1_cont bp (MLast.CtAcc loc ct1 ct2) strm__
  | [: `("", "(");
       ct2 = class_signature_0
         ? P.error "class_signature" "'('" "[class_signature]";
       `("", ")") ? P.error "class_signature" "[class_signature]" "')'" :] ->
      let loc = P.gloc bp strm__ in
      let ct1 = a__ in
      class_signature_1_cont bp (MLast.CtApp loc ct1 ct2) strm__
  | [: :] -> a__ ]
and class_signature_2 =
  parser bp
  [ [: i = P.vala (P.token ("LIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.CtIde loc i
  | [: i = P.vala (P.token ("UIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.CtIde loc i ]
and class_signature_gen_cont bp a__ strm__ =
  let x = class_signature_1_cont bp a__ strm__ in
  if x == a__ then x else class_signature_gen_cont bp x strm__
and class_type_parameters_0 =
  parser bp
  [ [: `("", "[");
       tpl = P.vala (P.list1sep type_parameter_0 (P.token ("", ",")) False)
         ? P.error "class_type_parameters" "'['" "???";
       `("", "]") ? P.error "class_type_parameters" "???" "']'" :] ->
      let loc = P.gloc bp strm__ in
      (loc, tpl)
  | [: :] ->
      let loc = P.gloc bp strm__ in
      (loc, Ploc.VaVal []) ]
and class_type_declaration_0 =
  parser bp
    [: vf = P.vala (P.bool (P.token ("", "virtual")));
       ctp = class_type_parameters_0
         ? P.error "class_type_declaration" "???"
             "[class_type_parameters]" :] ->
      match Stream.npeek 2 strm__ with
      [ [("LIDENT", n); ("", "=")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let n = Ploc.VaVal n in
          match
            try Some (class_signature_0 strm__) with
            [ Stream.Failure -> None ]
          with
          [ Some cs ->
              let loc = P.gloc bp strm__ in
              {MLast.ciLoc = loc; MLast.ciVir = vf; MLast.ciPrm = ctp;
               MLast.ciNam = n; MLast.ciExp = cs}
          | _ ->
              raise
                (Stream.Error
                   (P.error "class_type_declaration" "lowercase identifier"
                      "[class_signature]")) ]
        }
      | _ ->
          raise
            (Stream.Error
               (P.error "class_type_declaration" "[class_type_parameters]"
                  "lowercase identifier")) ]
and str_item_0 =
  parser bp
  [ [: `("", "class") :] ->
      try
        match strm__ with parser
        [ [: `("", "type");
             ctd =
               P.vala
                 (P.list1sep class_type_declaration_0 (P.token ("", "and"))
                    False)
               ? P.error "str_item" "'type'" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.StClt loc ctd
        | [: cd =
               P.vala
                 (P.list1sep class_declaration_0 (P.token ("", "and"))
                    False) :] ->
            let loc = P.gloc bp strm__ in
            MLast.StCls loc cd ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "str_item" "'class'" "'type' or ???")) ]
  | [: `("", "exception");
       (_, c, tl, _) = constructor_declaration_0
         ? P.error "str_item" "'exception'" "[constructor_declaration]";
       b = rebind_exn_0
         ? P.error "str_item" "[constructor_declaration]" "[rebind_exn]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.StExc loc c tl b
  | [: `("", "external") :] ->
      try
        match strm__ with parser
        [ [: `("", "(");
             i = operator_rparen
               ? P.error "str_item" "'('" "[operator_rparen]";
             `("", ":") ? P.error "str_item" "[operator_rparen]" "':'";
             t = ctyp_0 ? P.error "str_item" "':'" "[ctyp]";
             `("", "=") ? P.error "str_item" "[ctyp]" "'='";
             pd = P.vala (P.list1 (P.token ("STRING", "")))
               ? P.error "str_item" "'='" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.StExt loc (Ploc.VaVal i) t pd
        | [: :] ->
            match Stream.npeek 2 strm__ with
            [ [("LIDENT", i); ("", ":")] -> do {
                Stream.junk strm__;
                Stream.junk strm__;
                let i = Ploc.VaVal i in
                match
                  try Some (ctyp_0 strm__) with [ Stream.Failure -> None ]
                with
                [ Some t ->
                    match strm__ with parser
                    [ [: `("", "=");
                         pd = P.vala (P.list1 (P.token ("STRING", "")))
                           ? P.error "str_item" "'='" "???" :] ->
                        let loc = P.gloc bp strm__ in
                        MLast.StExt loc i t pd
                    | [: :] ->
                        raise
                          (Stream.Error (P.error "str_item" "[ctyp]" "'='")) ]
                | _ ->
                    raise
                      (Stream.Error
                         (P.error "str_item" "lowercase identifier"
                            "[ctyp]")) ]
              }
            | _ -> raise Stream.Failure ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "str_item" "'external'"
                  "'(' or lowercase identifier")) ]
  | [: `("", "include");
       me = module_expr_0
         ? P.error "str_item" "'include'" "[module_expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.StInc loc me
  | [: `("", "module") :] ->
      try
        match strm__ with parser
        [ [: `("", "type");
             i = P.vala ident_0 ? P.error "str_item" "'type'" "[ident]";
             `("", "=") ? P.error "str_item" "[ident]" "'='";
             mt = module_type_0
               ? P.error "str_item" "'='" "[module_type]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.StMty loc i mt
        | [: r = P.vala (P.bool (P.token ("", "rec")));
             l = P.vala (P.list1sep mod_binding_0 (P.token ("", "and")) False)
               ? P.error "str_item" "???" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.StMod loc r l ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "str_item" "'module'" "'type' or ???")) ]
  | [: `("", "open");
       i = P.vala mod_ident_0
         ? P.error "str_item" "'open'" "[mod_ident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.StOpn loc i
  | [: `("", "type");
       nr = P.vala (P.bool (P.token ("", "nonrec")))
         ? P.error "str_item" "'type'" "???";
       tdl =
         P.vala (P.list1sep type_declaration_0 (P.token ("", "and")) False)
         ? P.error "str_item" "???" "???" :] ->
      let loc = P.gloc bp strm__ in
      MLast.StTyp loc nr tdl
  | [: `("", "let") :] ->
      try
        match strm__ with parser
        [ [: `("", "module");
             m = P.vala uidopt_0 ? P.error "str_item" "'module'" "[uidopt]";
             mb = mod_fun_binding_0
               ? P.error "str_item" "[uidopt]" "[mod_fun_binding]";
             `("", "in") ? P.error "str_item" "[mod_fun_binding]" "'in'";
             e = expr_0 ? P.error "str_item" "'in'" "[expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.StExp loc (MLast.ExLmd loc m mb e)
        | [: `("", "open");
             m = module_expr_0 ? P.error "str_item" "'open'" "[module_expr]";
             `("", "in") ? P.error "str_item" "[module_expr]" "'in'";
             e = expr_0 ? P.error "str_item" "'in'" "[expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.StExp loc (MLast.ExLop loc m e)
        | [: r = P.vala (P.bool (P.token ("", "rec")));
             l = P.vala (P.list1sep let_binding_0 (P.token ("", "and")) False)
               ? P.error "str_item" "???" "???" :] ->
            try
              match strm__ with parser
              [ [: `("", "in");
                   x = expr_0 ? P.error "str_item" "'in'" "[expr]" :] ->
                  let loc = P.gloc bp strm__ in
                  let e = MLast.ExLet loc r l x in
                  MLast.StExp loc e
              | [: :] ->
                  let loc = P.gloc bp strm__ in
                  match l with
                  [ Ploc.VaVal [(p, e)] ->
                      match p with
                      [ MLast.PaAny _ -> MLast.StExp loc e
                      | _ -> MLast.StVal loc r l ]
                  | _ -> MLast.StVal loc r l ] ]
            with
            [ Stream.Failure ->
                raise (Stream.Error (P.error "str_item" "???" "'in'")) ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "str_item" "'let'" "'module' or 'open' or ???")) ]
  | [: e = expr_0 :] ->
      let loc = P.gloc bp strm__ in
      MLast.StExp loc e ]
and structure_0 =
  parser bp
    [: st =
         P.vala
           (P.list0
              (parser
                 [: s = str_item_0 :] ->
                   let _ =
                     try P.option (P.token ("", ";;")) strm__ with
                     [ Stream.Failure -> raise (Stream.Error "") ]
                   in
                   s)) :] ->
      st
and module_expr_0 =
  parser bp
  [ [: `("", "functor");
       arg = P.vala functor_parameter_0
         ? P.error "module_expr" "'functor'" "[functor_parameter]";
       `("", "->") ? P.error "module_expr" "[functor_parameter]" "'->'";
       me = module_expr_0 ? P.error "module_expr" "'->'" "[module_expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MeFun loc arg me
  | [: `("", "struct");
       st = structure_0 ? P.error "module_expr" "'struct'" "[structure]";
       `("", "end") ? P.error "module_expr" "[structure]" "'end'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MeStr loc st
  | [: :] -> module_expr_1 strm__ ]
and module_expr_1 =
  parser bp
    [: :] ->
      let a = module_expr_2 strm__ in
      module_expr_1_cont bp a strm__
and module_expr_1_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       me2 = P.orzero module_expr_2 module_expr_0
         ? P.error "module_expr" "'.'" "[module_expr]" :] ->
      let loc = P.gloc bp strm__ in
      let me1 = a__ in
      module_expr_1_cont bp (MLast.MeAcc loc me1 me2) strm__
  | [: :] -> a__ ]
and module_expr_2 =
  parser bp
    [: :] ->
      let a = module_expr_3 strm__ in
      module_expr_2_cont bp a strm__
and module_expr_2_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "(") :] ->
      try
        match strm__ with parser
        [ [: `("", ")") :] ->
            let loc = P.gloc bp strm__ in
            let me1 = a__ in
            module_expr_2_cont bp
              (MLast.MeApp loc me1 (MLast.MeStr loc (Ploc.VaVal []))) strm__
        | [: me2 = module_expr_0;
             `("", ")") ? P.error "module_expr" "[module_expr]" "')'" :] ->
            let loc = P.gloc bp strm__ in
            let me1 = a__ in
            module_expr_2_cont bp (MLast.MeApp loc me1 me2) strm__ ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "module_expr" "'('" "')' or [module_expr]")) ]
  | [: :] -> a__ ]
and module_expr_3 =
  parser bp
  [ [: `("", "(") :] ->
      try
        match strm__ with parser
        [ [: `("", "val");
             e = expr_0 ? P.error "module_expr" "'val'" "[expr]" :] ->
            try
              match strm__ with parser
              [ [: `("", ":");
                   mt = module_type_0
                     ? P.error "module_expr" "':'" "[module_type]";
                   `("", ")")
                     ? P.error "module_expr" "[module_type]" "')'" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.MeUnp loc e (Some mt)
              | [: `("", ")") :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.MeUnp loc e None ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "module_expr" "[expr]" "':' or ')'")) ]
        | [: me = module_expr_0 :] ->
            try
              match strm__ with parser
              [ [: `("", ":");
                   mt = module_type_0
                     ? P.error "module_expr" "':'" "[module_type]";
                   `("", ")")
                     ? P.error "module_expr" "[module_type]" "')'" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.MeTyc loc me mt
              | [: `("", ")") :] -> me ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "module_expr" "[module_expr]" "':' or ')'")) ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "module_expr" "'('" "'val' or [module_expr]")) ]
  | [: i = mod_expr_ident_0 :] -> i ]
and module_expr_gen_cont bp a__ strm__ =
  let x = module_expr_1_cont bp a__ strm__ in
  let x = module_expr_2_cont bp x strm__ in
  if x == a__ then x else module_expr_gen_cont bp x strm__
and typevar_0 =
  parser bp
    [: `("", "'"); i = ident_0 ? P.error "typevar" "'''" "[ident]" :] -> i
and poly_type_0 =
  parser bp
  [ [: `("", "type");
       nt = P.list1 (P.token ("LIDENT", ""))
         ? P.error "poly_type" "'type'" "lowercase identifier";
       `("", ".") ? P.error "poly_type" "" "lowercase identifier or '.'";
       ct = ctyp_0 ? P.error "poly_type" "'.'" "[ctyp]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyPot loc (Ploc.VaVal nt) ct
  | [: _ = test_typevar_list_dot;
       tpl = P.list1 typevar_0
         ? P.error "poly_type" "[test_typevar_list_dot]" "[typevar]";
       `("", ".") ? P.error "poly_type" "" "[typevar] or '.'";
       t2 = ctyp_0 ? P.error "poly_type" "'.'" "[ctyp]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyPol loc (Ploc.VaVal tpl) t2
  | [: t = ctyp_0 :] -> t ]
and field_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("LIDENT", lab); ("", ":")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          match
            try Some (poly_type_0 strm__) with [ Stream.Failure -> None ]
          with
          [ Some t -> (lab, t)
          | _ ->
              raise
                (Stream.Error
                   (P.error "field" "lowercase identifier" "[poly_type]")) ]
        }
      | _ -> raise Stream.Failure ]
and meth_list_0 =
  parser bp
    [: f = field_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (meth_list_0 strm__) with [ Stream.Failure -> None ]
              with
              [ Some ml -> [f :: ml]
              | _ -> [f] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error (P.error "meth_list" "';'" "[meth_list]")) ]
        | [: :] -> [f] ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "meth_list" "[field]" "';'")) ]
and class_longident_0 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("UIDENT", m); ("", ".")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          match
            try Some (class_longident_0 strm__) with
            [ Stream.Failure -> None ]
          with
          [ Some l -> [m :: l]
          | _ ->
              raise
                (Stream.Error
                   (P.error "class_longident" "uppercase identifier"
                      "[class_longident]")) ]
        }
      | _ -> match strm__ with parser [: `("LIDENT", i) :] -> [i] ]
and name_tag_0 =
  parser bp
    [: `("", "`"); i = ident_0 ? P.error "name_tag" "'`'" "[ident]" :] -> i
and poly_variant_0 =
  parser bp
  [ [: `("", "`");
       i = P.vala ident_0 ? P.error "poly_variant" "'`'" "[ident]" :] ->
      try
        match strm__ with parser
        [ [: `("", "of");
             ao = P.vala (P.bool (P.token ("", "&")))
               ? P.error "poly_variant" "'of'" "???";
             l = P.vala (P.list1sep ctyp_0 (P.token ("", "&")) False)
               ? P.error "poly_variant" "???" "???" :] ->
            let loc = P.gloc bp strm__ in
            MLast.PvTag loc i ao l
        | [: :] ->
            let loc = P.gloc bp strm__ in
            MLast.PvTag loc i (Ploc.VaVal True) (Ploc.VaVal []) ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "poly_variant" "[ident]" "'of'")) ]
  | [: t = ctyp_0 :] ->
      let loc = P.gloc bp strm__ in
      MLast.PvInh loc t ]
and ctyp_0 =
  parser bp
    [: :] ->
      let a = ctyp_1 strm__ in
      ctyp_gen_cont bp a strm__
and ctyp_0_cont bp a__ strm__ =
  match Stream.npeek 2 strm__ with
  [ [("", "as"); ("", "'")] -> do {
      Stream.junk strm__;
      Stream.junk strm__;
      match try Some (ident_0 strm__) with [ Stream.Failure -> None ] with
      [ Some i ->
          let loc = P.gloc bp strm__ in
          let t1 = a__ in
          ctyp_0_cont bp (MLast.TyAli loc t1 (MLast.TyQuo loc (Ploc.VaVal i)))
            strm__
      | _ -> raise (Stream.Error (P.error "ctyp" "'as'" "[ident]")) ]
    }
  | _ -> a__ ]
and ctyp_1 =
  parser bp
    [: :] ->
      let a = ctyp_2 strm__ in
      ctyp_1_cont bp a strm__
and ctyp_1_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "->");
       t2 = P.orzero ctyp_1 ctyp_0 ? P.error "ctyp" "'->'" "[ctyp]" :] ->
      let t1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.TyArr loc t1 t2
  | [: :] -> a__ ]
and ctyp_2 =
  parser bp
    [: :] ->
      match Stream.npeek 2 strm__ with
      [ [("LIDENT", i); ("", ":")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          let i = Ploc.VaVal i in
          match
            try Some (P.orzero ctyp_2 ctyp_0 strm__) with
            [ Stream.Failure -> None ]
          with
          [ Some t ->
              let loc = P.gloc bp strm__ in
              MLast.TyLab loc i t
          | _ ->
              raise
                (Stream.Error
                   (P.error "ctyp" "lowercase identifier" "[ctyp]")) ]
        }
      | _ ->
          match
            try Some (P.vala (P.token ("QUESTIONIDENTCOLON", "")) strm__) with
            [ Stream.Failure -> None ]
          with
          [ Some i ->
              match
                try Some (P.orzero ctyp_2 ctyp_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some t ->
                  let loc = P.gloc bp strm__ in
                  MLast.TyOlb loc i t
              | _ ->
                  raise
                    (Stream.Error
                       (P.error "ctyp" "QUESTIONIDENTCOLON" "[ctyp]")) ]
          | _ ->
              match Stream.npeek 2 strm__ with
              [ [("QUESTIONIDENT", i); ("", ":")] -> do {
                  Stream.junk strm__;
                  Stream.junk strm__;
                  let i = Ploc.VaVal i in
                  match
                    try Some (P.orzero ctyp_2 ctyp_0 strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some t ->
                      let loc = P.gloc bp strm__ in
                      MLast.TyOlb loc i t
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "ctyp" "QUESTIONIDENT" "[ctyp]")) ]
                }
              | _ -> ctyp_3 strm__ ] ] ]
and ctyp_3 =
  parser bp
    [: :] ->
      let a = ctyp_4 strm__ in
      ctyp_3_cont bp a strm__
and ctyp_3_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "*");
       tl = P.list1sep ctyp_4 (P.token ("", "*")) False
         ? P.error "ctyp" "'*'" "[ctyp level apply]" :] ->
      let loc = P.gloc bp strm__ in
      let t = a__ in
      ctyp_3_cont bp (MLast.TyTup loc (Ploc.VaVal [t :: tl])) strm__
  | [: :] -> a__ ]
and ctyp_4 =
  parser bp
    [: :] ->
      let a = ctyp_5 strm__ in
      ctyp_4_cont bp a strm__
and ctyp_4_cont bp a__ strm__ =
  match try Some (ctyp_5 strm__) with [ Stream.Failure -> None ] with
  [ Some t2 ->
      let loc = P.gloc bp strm__ in
      let t1 = a__ in
      ctyp_4_cont bp (MLast.TyApp loc t2 t1) strm__
  | _ -> a__ ]
and ctyp_5 =
  parser bp
    [: :] ->
      let a = ctyp_6 strm__ in
      ctyp_5_cont bp a strm__
and ctyp_5_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       t2 = P.orzero ctyp_6 ctyp_0 ? P.error "ctyp" "'.'" "[ctyp]" :] ->
      let loc = P.gloc bp strm__ in
      let t1 = a__ in
      ctyp_5_cont bp (MLast.TyAcc loc t1 t2) strm__
  | [: `("", "("); t2 = ctyp_0 ? P.error "ctyp" "'('" "[ctyp]";
       `("", ")") ? P.error "ctyp" "[ctyp]" "')'" :] ->
      let loc = P.gloc bp strm__ in
      let t1 = a__ in
      ctyp_5_cont bp (MLast.TyApp loc t1 t2) strm__
  | [: :] -> a__ ]
and ctyp_6 =
  parser bp
  [ [: `("", "[") :] ->
      try
        match strm__ with parser
        [ [: `("", ">") :] ->
            try
              match strm__ with parser
              [ [: `("", "]") :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.TyVrn loc (Ploc.VaVal []) (Some None)
              | [: :] ->
                  let _ =
                    try P.option (P.token ("", "|")) strm__ with
                    [ Stream.Failure -> raise (Stream.Error "") ]
                  in
                  match
                    try
                      Some
                        (P.vala
                           (P.list1sep poly_variant_0 (P.token ("", "|"))
                              False)
                           strm__)
                    with
                    [ Stream.Failure -> None ]
                  with
                  [ Some rfl ->
                      match strm__ with parser
                      [ [: `("", "]") :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.TyVrn loc rfl (Some None)
                      | [: :] ->
                          raise (Stream.Error (P.error "ctyp" "???" "']'")) ]
                  | _ -> raise (Stream.Error (P.error "ctyp" "" "???")) ] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error (P.error "ctyp" "'>'" "']' or '|' or ???")) ]
        | [: :] ->
            let _ =
              try P.option (P.token ("", "|")) strm__ with
              [ Stream.Failure -> raise (Stream.Error "") ]
            in
            match
              try
                Some
                  (P.vala
                     (P.list1sep poly_variant_0 (P.token ("", "|")) False)
                     strm__)
              with
              [ Stream.Failure -> None ]
            with
            [ Some rfl ->
                match strm__ with parser
                [ [: `("", "]") :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.TyVrn loc rfl None
                | [: :] -> raise (Stream.Error (P.error "ctyp" "???" "']'")) ]
            | _ -> raise (Stream.Error (P.error "ctyp" "" "???")) ] ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "ctyp" "'['" "'>' or '|' or ???")) ]
  | [: `("", "[<") :] ->
      let _ =
        try P.option (P.token ("", "|")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match
        try
          Some
            (P.vala (P.list1sep poly_variant_0 (P.token ("", "|")) False)
               strm__)
        with
        [ Stream.Failure -> None ]
      with
      [ Some rfl ->
          try
            match strm__ with parser
            [ [: `("", "]") :] ->
                let loc = P.gloc bp strm__ in
                MLast.TyVrn loc rfl (Some (Some (Ploc.VaVal [])))
            | [: `("", ">");
                 ntl = P.vala (P.list1 name_tag_0)
                   ? P.error "ctyp" "'>'" "???";
                 `("", "]") ? P.error "ctyp" "???" "']'" :] ->
                let loc = P.gloc bp strm__ in
                MLast.TyVrn loc rfl (Some (Some ntl)) ]
          with
          [ Stream.Failure ->
              raise (Stream.Error (P.error "ctyp" "???" "']' or '>'")) ]
      | _ -> raise (Stream.Error (P.error "ctyp" "" "???")) ]
  | [: `("", "#");
       id = P.vala class_longident_0
         ? P.error "ctyp" "'#'" "[class_longident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyCls loc id
  | [: `("", "<") :] ->
      try
        match Stream.npeek 2 strm__ with
        [ [("", ".."); ("", ">")] -> do {
            Stream.junk strm__;
            Stream.junk strm__;
            try
              let loc = P.gloc bp strm__ in
              MLast.TyObj loc (Ploc.VaVal []) (Ploc.VaVal True)
            with
            [ Stream.Failure ->
                raise (Stream.Error (P.error "ctyp" "'..'" "???")) ]
          }
        | _ ->
            match strm__ with parser
            [ [: `("", ">") :] ->
                let loc = P.gloc bp strm__ in
                MLast.TyObj loc (Ploc.VaVal []) (Ploc.VaVal False)
            | [: ml = P.vala meth_list_0;
                 v = P.vala (P.bool (P.token ("", "..")))
                   ? P.error "ctyp" "[meth_list]" "???";
                 `("", ">") ? P.error "ctyp" "???" "'>'" :] ->
                let loc = P.gloc bp strm__ in
                MLast.TyObj loc ml v ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "ctyp" "'<'" "'..' or '>' or [meth_list]")) ]
  | [: `("", "'"); i = P.vala ident_0 ? P.error "ctyp" "'''" "[ident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyQuo loc i
  | [: `("", "_") :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyAny loc
  | [: `("", "(") :] ->
      try
        match strm__ with parser
        [ [: `("", "module");
             mt = module_type_0 ? P.error "ctyp" "'module'" "[module_type]";
             `("", ")") ? P.error "ctyp" "[module_type]" "')'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.TyPck loc mt
        | [: t = ctyp_0 :] ->
            try
              match strm__ with parser
              [ [: `("", ",");
                   tl = P.list1sep ctyp_0 (P.token ("", ",")) False
                     ? P.error "ctyp" "','" "[ctyp]";
                   `("", ")") ? P.error "ctyp" "" "[ctyp] or ')'";
                   i = ctyp_5
                     ? P.error "ctyp" "')'" "[ctyp level ctyp2]" :] ->
                  let loc = P.gloc bp strm__ in
                  List.fold_left (fun c a -> MLast.TyApp loc c a) i [t :: tl]
              | [: `("", ")") :] -> t ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error (P.error "ctyp" "[ctyp]" "',' or ')'")) ] ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "ctyp" "'('" "'module' or [ctyp]")) ]
  | [: i = P.vala (P.token ("LIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyLid loc i
  | [: i = P.vala (P.token ("UIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.TyUid loc i ]
and ctyp_gen_cont bp a__ strm__ =
  let x = ctyp_0_cont bp a__ strm__ in
  let x = ctyp_1_cont bp x strm__ in
  let x = ctyp_3_cont bp x strm__ in
  let x = ctyp_4_cont bp x strm__ in
  let x = ctyp_5_cont bp x strm__ in
  if x == a__ then x else ctyp_gen_cont bp x strm__
and simple_type_parameter_0 =
  parser bp
  [ [: `("", "'");
       i = ident_0 ? P.error "simple_type_parameter" "'''" "[ident]" :] ->
      Some i
  | [: `("", "_") :] -> None ]
and type_parameter_0 =
  parser bp
  [ [: `("", "+");
       p = P.vala simple_type_parameter_0
         ? P.error "type_parameter" "'+'" "[simple_type_parameter]" :] ->
      (p, Some True)
  | [: `("", "-");
       p = P.vala simple_type_parameter_0
         ? P.error "type_parameter" "'-'" "[simple_type_parameter]" :] ->
      (p, Some False)
  | [: p = P.vala simple_type_parameter_0 :] -> (p, None) ]
and type_parameters_0 =
  parser bp
  [ [: `("", "(");
       tpl = P.list1sep type_parameter_0 (P.token ("", ",")) False
         ? P.error "type_parameters" "'('" "[type_parameter]";
       `("", ")")
         ? P.error "type_parameters" "" "[type_parameter] or ')'" :] ->
      tpl
  | [: tp = type_parameter_0 :] -> [tp]
  | [: :] -> [] ]
and with_constr_0 =
  parser bp
  [ [: `("", "type");
       tpl = P.vala type_parameters_0
         ? P.error "with_constr" "'type'" "[type_parameters]";
       i = P.vala mod_ident_0
         ? P.error "with_constr" "[type_parameters]" "[mod_ident]" :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             pf = P.vala (P.bool (P.token ("", "private")))
               ? P.error "with_constr" "'='" "???";
             t = ctyp_0 ? P.error "with_constr" "???" "[ctyp]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.WcTyp loc i tpl pf t
        | [: `("", ":=");
             t = ctyp_0 ? P.error "with_constr" "':='" "[ctyp]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.WcTys loc i tpl t ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "with_constr" "[mod_ident]" "'=' or ':='")) ]
  | [: `("", "module");
       i = P.vala mod_ident_0
         ? P.error "with_constr" "'module'" "[mod_ident]" :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             me = module_expr_0
               ? P.error "with_constr" "'='" "[module_expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.WcMod loc i me
        | [: `("", ":=");
             me = module_expr_0
               ? P.error "with_constr" "':='" "[module_expr]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.WcMos loc i me ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "with_constr" "[mod_ident]" "'=' or ':='")) ] ]
and functor_parameter_0 =
  parser bp
    [: `("", "(") :] ->
      try
        match strm__ with parser
        [ [: `("", ")") :] -> None
        | [: i = P.vala uidopt_0;
             `("", ":") ? P.error "functor_parameter" "[uidopt]" "':'";
             t = module_type_0
               ? P.error "functor_parameter" "':'" "[module_type]";
             `("", ")")
               ? P.error "functor_parameter" "[module_type]" "')'" :] ->
            Some (i, t) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "functor_parameter" "'('" "')' or [uidopt]")) ]
and module_type_0 =
  parser bp
  [ [: `("", "functor");
       arg = P.vala functor_parameter_0
         ? P.error "module_type" "'functor'" "[functor_parameter]";
       `("", "->") ? P.error "module_type" "[functor_parameter]" "'->'";
       mt = module_type_0 ? P.error "module_type" "'->'" "[module_type]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MtFun loc arg mt
  | [: :] -> module_type_1 strm__ ]
and module_type_1 =
  parser bp
    [: :] ->
      let a = module_type_2 strm__ in
      module_type_1_cont bp a strm__
and module_type_1_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "->");
       mt2 = P.orzero module_type_1 module_type_0
         ? P.error "module_type" "'->'" "[module_type]" :] ->
      let mt1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.MtFun loc (Ploc.VaVal (Some (Ploc.VaVal None, mt1))) mt2
  | [: :] -> a__ ]
and module_type_2 =
  parser bp
    [: :] ->
      let a = module_type_3 strm__ in
      module_type_2_cont bp a strm__
and module_type_2_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "with");
       wcl = P.vala (P.list1sep with_constr_0 (P.token ("", "and")) False)
         ? P.error "module_type" "'with'" "???" :] ->
      let loc = P.gloc bp strm__ in
      let mt = a__ in
      module_type_2_cont bp (MLast.MtWit loc mt wcl) strm__
  | [: :] -> a__ ]
and module_type_3 =
  parser bp
  [ [: `("", "sig");
       sg = signature_0 ? P.error "module_type" "'sig'" "[signature]";
       `("", "end") ? P.error "module_type" "[signature]" "'end'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.MtSig loc sg
  | [: :] ->
      match Stream.npeek 3 strm__ with
      [ [("", "module"); ("", "type"); ("", "of")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          Stream.junk strm__;
          match
            try Some (module_expr_0 strm__) with [ Stream.Failure -> None ]
          with
          [ Some me ->
              let loc = P.gloc bp strm__ in
              MLast.MtTyo loc me
          | _ ->
              raise
                (Stream.Error
                   (P.error "module_type" "'module'" "[module_expr]")) ]
        }
      | _ ->
          match strm__ with parser
          [ [: `("", "(");
               mt = module_type_0
                 ? P.error "module_type" "'('" "[module_type]";
               `("", ")") ? P.error "module_type" "[module_type]" "')'" :] ->
              mt
          | [: i = mod_type_ident_0 :] -> i ] ] ]
and module_type_gen_cont bp a__ strm__ =
  let x = module_type_1_cont bp a__ strm__ in
  let x = module_type_2_cont bp x strm__ in
  if x == a__ then x else module_type_gen_cont bp x strm__
and uidopt_0 =
  parser bp
  [ [: `("", "_") :] -> None
  | [: m = P.vala (P.token ("UIDENT", "")) :] -> Some m ]
and patt_label_ident_0 =
  parser bp
    [: :] ->
      let a = patt_label_ident_1 strm__ in
      patt_label_ident_gen_cont bp a strm__
and patt_label_ident_0_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       p2 = P.orzero patt_label_ident_1 patt_label_ident_0
         ? P.error "patt_label_ident" "'.'" "[patt_label_ident]" :] ->
      let loc = P.gloc bp strm__ in
      let p1 = a__ in
      patt_label_ident_0_cont bp (MLast.PaAcc loc p1 p2) strm__
  | [: :] -> a__ ]
and patt_label_ident_1 =
  parser bp
  [ [: `("UIDENT", i) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaUid loc (Ploc.VaVal i)
  | [: `("LIDENT", i) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaLid loc (Ploc.VaVal i) ]
and patt_label_ident_gen_cont bp a__ strm__ =
  let x = patt_label_ident_0_cont bp a__ strm__ in
  if x == a__ then x else patt_label_ident_gen_cont bp x strm__
and lbl_patt_0 =
  parser bp
  [ [: `("", "_") :] ->
      let loc = P.gloc bp strm__ in
      (MLast.PaAny loc, MLast.PaAny loc)
  | [: i = patt_label_ident_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "="); p = patt_0 ? P.error "lbl_patt" "'='" "[patt]" :] ->
            (i, p)
        | [: :] -> (i, i) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "lbl_patt" "[patt_label_ident]" "'='")) ] ]
and lbl_patt_list_0 =
  parser bp
    [: le = lbl_patt_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (lbl_patt_list_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some lel -> [le :: lel]
              | _ -> [le] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "lbl_patt_list" "';'" "[lbl_patt_list]")) ]
        | [: :] -> [le] ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "lbl_patt_list" "[lbl_patt]" "';'")) ]
and patt_semi_list_0 =
  parser bp
    [: p = patt_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (patt_semi_list_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some pl -> [p :: pl]
              | _ -> [p] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "patt_semi_list" "';'" "[patt_semi_list]")) ]
        | [: :] -> [p] ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "patt_semi_list" "[patt]" "';'")) ]
and mod_ident_0 =
  parser bp
  [ [: `("LIDENT", i) :] -> [i]
  | [: `("UIDENT", i) :] ->
      try
        match strm__ with parser
        [ [: `("", ".");
             j = mod_ident_0 ? P.error "mod_ident" "'.'" "[mod_ident]" :] ->
            [i :: j]
        | [: :] -> [i] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "mod_ident" "uppercase identifier" "'.'")) ] ]
and ident_0 =
  parser bp
  [ [: `("LIDENT", i) :] -> i
  | [: `("UIDENT", i) :] -> i ]
and patt_0 =
  parser bp
    [: :] ->
      let a = patt_1 strm__ in
      patt_gen_cont bp a strm__
and patt_0_cont bp a__ strm__ =
  match Stream.npeek 2 strm__ with
  [ [("", "as"); ("LIDENT", i)] -> do {
      Stream.junk strm__;
      Stream.junk strm__;
      try
        let loc = P.gloc bp strm__ in
        let p1 = a__ in
        patt_0_cont bp (MLast.PaAli loc p1 (MLast.PaLid loc (Ploc.VaVal i)))
          strm__
      with
      [ Stream.Failure -> raise (Stream.Error (P.error "patt" "'as'" "???")) ]
    }
  | _ -> a__ ]
and patt_1 =
  parser bp
    [: :] ->
      let a = patt_2 strm__ in
      patt_1_cont bp a strm__
and patt_1_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "|");
       p2 = P.orzero patt_2 patt_0 ? P.error "patt" "'|'" "[patt]" :] ->
      let loc = P.gloc bp strm__ in
      let p1 = a__ in
      patt_1_cont bp (MLast.PaOrp loc p1 p2) strm__
  | [: :] -> a__ ]
and patt_2 =
  parser bp
    [: :] ->
      let a = patt_3 strm__ in
      patt_2_cont bp a strm__
and patt_2_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ",");
       pl = P.list1sep (P.orzero patt_3 patt_0) (P.token ("", ",")) False
         ? P.error "patt" "','" "[patt]" :] ->
      let loc = P.gloc bp strm__ in
      let p = a__ in
      patt_2_cont bp (MLast.PaTup loc (Ploc.VaVal [p :: pl])) strm__
  | [: :] -> a__ ]
and patt_3 =
  parser bp
    [: :] ->
      let a = patt_4 strm__ in
      patt_3_cont bp a strm__
and patt_3_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "..");
       p2 = P.orzero patt_4 patt_0 ? P.error "patt" "'..'" "[patt]" :] ->
      let p1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.PaRng loc p1 p2
  | [: :] -> a__ ]
and patt_4 =
  parser bp
    [: :] ->
      let a = patt_5 strm__ in
      patt_4_cont bp a strm__
and patt_4_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "::");
       p2 = P.orzero patt_4 patt_0 ? P.error "patt" "'::'" "[patt]" :] ->
      let p1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.PaApp loc (MLast.PaApp loc (MLast.PaUid loc (Ploc.VaVal "::")) p1)
        p2
  | [: :] -> a__ ]
and patt_5 =
  parser bp
  [ [: `("", "lazy");
       p = P.orzero patt_5 patt_0 ? P.error "patt" "'lazy'" "[patt]" :] ->
      let loc = P.gloc bp strm__ in
      patt_5_cont bp (MLast.PaLaz loc p) strm__
  | [: :] ->
      let a = patt_6 strm__ in
      patt_5_cont bp a strm__ ]
and patt_5_cont bp a__ strm__ =
  match try Some (patt_6 strm__) with [ Stream.Failure -> None ] with
  [ Some p2 ->
      let loc = P.gloc bp strm__ in
      let p1 = a__ in
      patt_5_cont bp
        (let (p1, p2) =
           match p1 with
           [ MLast.PaApp _ p11 p12 -> (p11, MLast.PaApp loc p12 p2)
           | _ -> (p1, p2) ]
         in
         match constr_patt_arity loc p1 with
         [ 1 -> MLast.PaApp loc p1 p2
         | n ->
             let p2 =
               match p2 with
               [ MLast.PaAny _ when n > 1 ->
                   let pl =
                     loop n where rec loop n =
                       if n = 0 then [] else [MLast.PaAny loc :: loop (n - 1)]
                   in
                   MLast.PaTup loc (Ploc.VaVal pl)
               | _ -> p2 ]
             in
             match p2 with
             [ MLast.PaTup _ (Ploc.VaVal pl) ->
                 List.fold_left (fun p1 p2 -> MLast.PaApp loc p1 p2) p1 pl
             | _ -> MLast.PaApp loc p1 p2 ] ])
        strm__
  | _ -> a__ ]
and patt_6 =
  parser bp
    [: :] ->
      let a = patt_7 strm__ in
      patt_6_cont bp a strm__
and patt_6_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ".");
       p2 = P.orzero patt_7 patt_0 ? P.error "patt" "'.'" "[patt]" :] ->
      let loc = P.gloc bp strm__ in
      let p1 = a__ in
      patt_6_cont bp (MLast.PaAcc loc p1 p2) strm__
  | [: :] -> a__ ]
and patt_7 =
  parser bp
  [ [: `("", "`"); s = P.vala ident_0 ? P.error "patt" "'`'" "[ident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaVrn loc s
  | [: `("", "#");
       t = P.vala mod_ident_0 ? P.error "patt" "'#'" "[mod_ident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaTyp loc t
  | [: `("", "-") :] ->
      try
        match strm__ with parser
        [ [: `("INT", s) :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaInt loc (Ploc.VaVal ("-" ^ s)) ""
        | [: `("INT_l", s) :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaInt loc (Ploc.VaVal ("-" ^ s)) "l"
        | [: `("INT_L", s) :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaInt loc (Ploc.VaVal ("-" ^ s)) "L"
        | [: `("INT_n", s) :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaInt loc (Ploc.VaVal ("-" ^ s)) "n"
        | [: `("FLOAT", s) :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaFlo loc (Ploc.VaVal ("-" ^ s)) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "patt" "'-'"
                  "integer or INT_l or INT_L or INT_n or float")) ]
  | [: `("UIDENT", "True") :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaUid loc (Ploc.VaVal "True_")
  | [: `("UIDENT", "False") :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaUid loc (Ploc.VaVal "False_")
  | [: `("", "false") :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaUid loc (Ploc.VaVal "False")
  | [: `("", "true") :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaUid loc (Ploc.VaVal "True")
  | [: `("", "[") :] ->
      try
        match strm__ with parser
        [ [: `("", "]") :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaUid loc (Ploc.VaVal "[]")
        | [: pl = patt_semi_list_0;
             `("", "]") ? P.error "patt" "[patt_semi_list]" "']'" :] ->
            let loc = P.gloc bp strm__ in
            mklistpat loc None pl ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "patt" "'['" "']' or [patt_semi_list]")) ]
  | [: `("", "[|") :] ->
      try
        match strm__ with parser
        [ [: `("", "|]") :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaArr loc (Ploc.VaVal [])
        | [: pl = P.vala patt_semi_list_0;
             `("", "|]") ? P.error "patt" "[patt_semi_list]" "'|]'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaArr loc pl ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "patt" "'[|'" "'|]' or [patt_semi_list]")) ]
  | [: `("", "{");
       lpl = P.vala lbl_patt_list_0 ? P.error "patt" "'{'" "[lbl_patt_list]";
       `("", "}") ? P.error "patt" "[lbl_patt_list]" "'}'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaRec loc lpl
  | [: `("", "(") :] ->
      try
        match strm__ with parser
        [ [: `("", ")") :] ->
            let loc = P.gloc bp strm__ in
            MLast.PaUid loc (Ploc.VaVal "()")
        | [: :] ->
            match Stream.npeek 3 strm__ with
            [ [("", "type"); ("LIDENT", s); ("", ")")] -> do {
                Stream.junk strm__;
                Stream.junk strm__;
                Stream.junk strm__;
                let s = Ploc.VaVal s in
                try
                  let loc = P.gloc bp strm__ in
                  MLast.PaNty loc s
                with
                [ Stream.Failure ->
                    raise (Stream.Error (P.error "patt" "'type'" "???")) ]
              }
            | _ ->
                match strm__ with parser
                [ [: `("", "module");
                     s = P.vala uidopt_0
                       ? P.error "patt" "'module'" "[uidopt]" :] ->
                    try
                      match strm__ with parser
                      [ [: `("", ":");
                           mt = module_type_0
                             ? P.error "patt" "':'" "[module_type]";
                           `("", ")")
                             ? P.error "patt" "[module_type]" "')'" :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.PaUnp loc s (Some mt)
                      | [: `("", ")") :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.PaUnp loc s None ]
                    with
                    [ Stream.Failure ->
                        raise
                          (Stream.Error
                             (P.error "patt" "[uidopt]" "':' or ')'")) ]
                | [: op = operator_rparen :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.PaLid loc (Ploc.VaVal op)
                | [: pl = P.vala p_phony;
                     `("", ")") ? P.error "patt" "[p_phony]" "')'" :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.PaTup loc pl
                | [: p = patt_0 :] ->
                    try
                      match strm__ with parser
                      [ [: `("", ":");
                           t = ctyp_0 ? P.error "patt" "':'" "[ctyp]";
                           `("", ")") ? P.error "patt" "[ctyp]" "')'" :] ->
                          let loc = P.gloc bp strm__ in
                          MLast.PaTyc loc p t
                      | [: `("", ")") :] -> p ]
                    with
                    [ Stream.Failure ->
                        raise
                          (Stream.Error
                             (P.error "patt" "[patt]" "':' or ')'")) ] ] ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "patt" "'('"
                  "')' or 'type' or 'module' or [operator_rparen] or [p_phony] or [patt]")) ]
  | [: `("", "_") :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaAny loc
  | [: s = P.vala (P.token ("LIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaLid loc s
  | [: s = P.vala (P.token ("UIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaUid loc s
  | [: s = P.vala (P.token ("INT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaInt loc s ""
  | [: s = P.vala (P.token ("INT_l", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaInt loc s "l"
  | [: s = P.vala (P.token ("INT_L", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaInt loc s "L"
  | [: s = P.vala (P.token ("INT_n", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaInt loc s "n"
  | [: s = P.vala (P.token ("FLOAT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaFlo loc s
  | [: s = P.vala (P.token ("STRING", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaStr loc s
  | [: s = P.vala (P.token ("CHAR", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaChr loc s
  | [: `("QUOTATION", x) :] ->
      let loc = P.gloc bp strm__ in
      let con = quotation_content x in
      Pcaml.handle_patt_quotation loc con
  | [: p = labeled_patt_0 :] -> p ]
and patt_gen_cont bp a__ strm__ =
  let x = patt_0_cont bp a__ strm__ in
  let x = patt_1_cont bp x strm__ in
  let x = patt_2_cont bp x strm__ in
  let x = patt_3_cont bp x strm__ in
  let x = patt_4_cont bp x strm__ in
  let x = patt_5_cont bp x strm__ in
  let x = patt_6_cont bp x strm__ in
  if x == a__ then x else patt_gen_cont bp x strm__
and stream_patt_comp_0 =
  parser bp
  [ [: `("", "'"); p = patt_0 ? P.error "stream_patt_comp" "'''" "[patt]";
       eo =
         P.vala
           (P.option
              (parser
                 [: `("", "when");
                    e = expr_1
                      ? P.error "implem" "'when'" "[expr level expr1]" :] ->
                   e))
         ? P.error "stream_patt_comp" "[patt]" "???" :] ->
      let loc = P.gloc bp strm__ in
      SpTrm loc p eo
  | [: `("", "?=");
       pll = P.list1sep lookahead_0 (P.token ("", "|")) False
         ? P.error "stream_patt_comp" "'?='" "[lookahead]" :] ->
      let loc = P.gloc bp strm__ in
      SpLhd loc pll
  | [: p = patt_0 :] ->
      try
        match strm__ with parser
        [ [: `("", "=");
             e = expr_1
               ? P.error "stream_patt_comp" "'='" "[expr level expr1]" :] ->
            let loc = P.gloc bp strm__ in
            SpNtr loc p e
        | [: :] ->
            let loc = P.gloc bp strm__ in
            SpStr loc p ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "stream_patt_comp" "[patt]" "'='")) ] ]
and stream_patt_0 =
  parser bp
  [ [: spc = stream_patt_comp_0 :] ->
      try
        match strm__ with parser
        [ [: `("", ";") :] ->
            try
              match
                try Some (stream_patt_kont_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some sp -> [(spc, SpoNoth) :: sp]
              | _ -> [(spc, SpoNoth)] ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "stream_patt" "';'" "[stream_patt_kont]")) ]
        | [: :] -> [(spc, SpoNoth)] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "stream_patt" "[stream_patt_comp]" "';'")) ]
  | [: spc = stream_patt_let_0;
       sp = stream_patt_0
         ? P.error "stream_patt" "[stream_patt_let]" "[stream_patt]" :] ->
      [spc :: sp]
  | [: :] -> [] ]
and parser_case_0 =
  parser bp
    [: `("", "[<");
       sp = stream_patt_0 ? P.error "parser_case" "'[<'" "[stream_patt]";
       `("", ">]") ? P.error "parser_case" "[stream_patt]" "'>]'" :] ->
      let po =
        try P.option ipatt_0 strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match strm__ with parser
      [ [: `("", "->");
           e = expr_0 ? P.error "parser_case" "'->'" "[expr]" :] ->
          (sp, po, e)
      | [: :] -> raise (Stream.Error (P.error "parser_case" "" "'->'")) ]
and ipatt_0 =
  parser bp
  [ [: `("", "_") :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaAny loc
  | [: `("LIDENT", i) :] ->
      let loc = P.gloc bp strm__ in
      MLast.PaLid loc (Ploc.VaVal i) ]
and expr_0 =
  parser bp
  [ [: el = P.vala e_phony :] ->
      let loc = P.gloc bp strm__ in
      expr_0_cont bp (MLast.ExSeq loc el) strm__
  | [: :] ->
      let a = expr_1 strm__ in
      expr_0_cont bp a strm__ ]
and expr_0_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ";") :] ->
      try
        match try Some (expr_0 strm__) with [ Stream.Failure -> None ] with
        [ Some e2 ->
            let e1 = a__ in
            let loc = P.gloc bp strm__ in
            MLast.ExSeq loc (Ploc.VaVal [e1 :: get_seq e2])
        | _ ->
            let e1 = a__ in
            e1 ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "expr" "';'" "[expr]")) ]
  | [: :] -> a__ ]
and expr_1 =
  parser bp
  [ [: `("", "parser") :] ->
      let po =
        try P.option ipatt_0 strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      let _ =
        try P.option (P.token ("", "|")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match
        try
          Some (P.list1sep parser_case_0 (P.token ("", "|")) False strm__)
        with
        [ Stream.Failure -> None ]
      with
      [ Some pcl ->
          let loc = P.gloc bp strm__ in
          cparser loc po pcl
      | _ -> raise (Stream.Error (P.error "expr" "" "[parser_case]")) ]
  | [: `("", "let") :] ->
      try
        match strm__ with parser
        [ [: `("", "module");
             m = P.vala uidopt_0 ? P.error "expr" "'module'" "[uidopt]";
             mb = mod_fun_binding_0
               ? P.error "expr" "[uidopt]" "[mod_fun_binding]";
             `("", "in") ? P.error "expr" "[mod_fun_binding]" "'in'";
             e = expr_0 ? P.error "expr" "'in'" "[expr level top]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExLmd loc m mb e
        | [: `("", "open");
             m = module_expr_0 ? P.error "expr" "'open'" "[module_expr]";
             `("", "in") ? P.error "expr" "[module_expr]" "'in'";
             e = expr_0 ? P.error "expr" "'in'" "[expr level top]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExLop loc m e
        | [: o = P.vala (P.bool (P.token ("", "rec")));
             l = P.vala (P.list1sep let_binding_0 (P.token ("", "and")) False)
               ? P.error "expr" "???" "???";
             `("", "in") ? P.error "expr" "???" "'in'";
             x = expr_0 ? P.error "expr" "'in'" "[expr level top]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExLet loc o l x ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'let'" "'module' or 'open' or ???")) ]
  | [: `("", "function") :] ->
      let _ =
        try P.option (P.token ("", "|")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match
        try
          Some
            (P.vala (P.list1sep match_case_0 (P.token ("", "|")) False)
               strm__)
        with
        [ Stream.Failure -> None ]
      with
      [ Some l ->
          let loc = P.gloc bp strm__ in
          MLast.ExFun loc l
      | _ -> raise (Stream.Error (P.error "expr" "" "???")) ]
  | [: `("", "fun") :] ->
      try
        match
          try Some (labeled_patt_0 strm__) with [ Stream.Failure -> None ]
        with
        [ Some p ->
            match
              try Some (fun_def_0 strm__) with [ Stream.Failure -> None ]
            with
            [ Some (eo, e) ->
                let loc = P.gloc bp strm__ in
                MLast.ExFun loc (Ploc.VaVal [(p, Ploc.VaVal eo, e)])
            | _ ->
                raise
                  (Stream.Error
                     (P.error "expr" "[labeled_patt]" "[fun_def]")) ]
        | _ ->
            match
              try Some (patt_7 strm__) with [ Stream.Failure -> None ]
            with
            [ Some p ->
                match
                  try Some (fun_def_0 strm__) with [ Stream.Failure -> None ]
                with
                [ Some (eo, e) ->
                    let loc = P.gloc bp strm__ in
                    MLast.ExFun loc (Ploc.VaVal [(p, Ploc.VaVal eo, e)])
                | _ ->
                    raise
                      (Stream.Error
                         (P.error "expr" "[patt level simple]" "[fun_def]")) ]
            | _ -> raise Stream.Failure ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'fun'"
                  "[labeled_patt] or [patt level simple]")) ]
  | [: `("", "match"); e = expr_0 ? P.error "expr" "'match'" "[expr]";
       `("", "with") ? P.error "expr" "[expr]" "'with'" :] ->
      try
        match strm__ with parser
        [ [: `("", "parser") :] ->
            let po =
              try P.option ipatt_0 strm__ with
              [ Stream.Failure -> raise (Stream.Error "") ]
            in
            let _ =
              try P.option (P.token ("", "|")) strm__ with
              [ Stream.Failure -> raise (Stream.Error "") ]
            in
            match
              try
                Some
                  (P.list1sep parser_case_0 (P.token ("", "|")) False strm__)
              with
              [ Stream.Failure -> None ]
            with
            [ Some pcl ->
                let loc = P.gloc bp strm__ in
                cparser_match loc e po pcl
            | _ -> raise (Stream.Error (P.error "expr" "" "[parser_case]")) ]
        | [: :] ->
            let _ =
              try P.option (P.token ("", "|")) strm__ with
              [ Stream.Failure -> raise (Stream.Error "") ]
            in
            match
              try
                Some
                  (P.vala (P.list1sep match_case_0 (P.token ("", "|")) False)
                     strm__)
              with
              [ Stream.Failure -> None ]
            with
            [ Some l ->
                let loc = P.gloc bp strm__ in
                MLast.ExMat loc e l
            | _ -> raise (Stream.Error (P.error "expr" "" "???")) ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'with'" "'parser' or '|' or ???")) ]
  | [: `("", "try"); e = expr_0 ? P.error "expr" "'try'" "[expr]";
       `("", "with") ? P.error "expr" "[expr]" "'with'" :] ->
      let _ =
        try P.option (P.token ("", "|")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      match
        try
          Some
            (P.vala (P.list1sep match_case_0 (P.token ("", "|")) False)
               strm__)
        with
        [ Stream.Failure -> None ]
      with
      [ Some l ->
          let loc = P.gloc bp strm__ in
          MLast.ExTry loc e l
      | _ -> raise (Stream.Error (P.error "expr" "" "???")) ]
  | [: `("", "if"); e1 = expr_0 ? P.error "expr" "'if'" "[expr]";
       `("", "then") ? P.error "expr" "[expr]" "'then'";
       e2 = expr_1 ? P.error "expr" "'then'" "[expr level expr1]" :] ->
      try
        match strm__ with parser
        [ [: `("", "else");
             e3 = expr_1 ? P.error "expr" "'else'" "[expr level expr1]" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExIfe loc e1 e2 e3
        | [: :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExIfe loc e1 e2 (MLast.ExUid loc (Ploc.VaVal "()")) ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "expr" "[expr level expr1]" "'else'")) ]
  | [: :] ->
      match Stream.npeek 3 strm__ with
      [ [("", "for"); ("LIDENT", i); ("", "=")] -> do {
          Stream.junk strm__;
          Stream.junk strm__;
          Stream.junk strm__;
          let i = Ploc.VaVal i in
          match try Some (expr_0 strm__) with [ Stream.Failure -> None ] with
          [ Some e1 ->
              match
                try Some (P.vala direction_flag_0 strm__) with
                [ Stream.Failure -> None ]
              with
              [ Some df ->
                  match
                    try Some (expr_0 strm__) with [ Stream.Failure -> None ]
                  with
                  [ Some e2 ->
                      match strm__ with parser
                      [ [: `("", "do");
                           e = P.vala expr_0 ? P.error "expr" "'do'" "[expr]";
                           `("", "done")
                             ? P.error "expr" "[expr]" "'done'" :] ->
                          let loc = P.gloc bp strm__ in
                          let el = Pcaml.vala_map get_seq e in
                          MLast.ExFor loc i e1 e2 df el
                      | [: :] ->
                          raise
                            (Stream.Error (P.error "expr" "[expr]" "'do'")) ]
                  | _ ->
                      raise
                        (Stream.Error
                           (P.error "expr" "[direction_flag]" "[expr]")) ]
              | _ ->
                  raise
                    (Stream.Error
                       (P.error "expr" "[expr]" "[direction_flag]")) ]
          | _ -> raise (Stream.Error (P.error "expr" "'for'" "[expr]")) ]
        }
      | _ ->
          match strm__ with parser
          [ [: `("", "while");
               e1 = expr_0 ? P.error "expr" "'while'" "[expr]";
               `("", "do") ? P.error "expr" "[expr]" "'do'";
               e2 = P.vala expr_0 ? P.error "expr" "'do'" "[expr]";
               `("", "done") ? P.error "expr" "[expr]" "'done'" :] ->
              let loc = P.gloc bp strm__ in
              let el = Pcaml.vala_map get_seq e2 in
              MLast.ExWhi loc e1 el
          | [: :] -> expr_2 strm__ ] ] ]
and expr_2 =
  parser bp
    [: :] ->
      let a = expr_3 strm__ in
      expr_2_cont bp a strm__
and expr_2_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ",");
       el = P.list1sep (P.orzero expr_3 expr_0) (P.token ("", ",")) False
         ? P.error "expr" "','" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e = a__ in
      expr_2_cont bp (MLast.ExTup loc (Ploc.VaVal [e :: el])) strm__
  | [: :] -> a__ ]
and expr_3 =
  parser bp
    [: :] ->
      let a = expr_4 strm__ in
      expr_3_cont bp a strm__
and expr_3_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", ":=");
       e2 = expr_1 ? P.error "expr" "':='" "[expr level expr1]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExAss loc
        (MLast.ExAcc loc e1 (MLast.ExLid loc (Ploc.VaVal "val"))) e2
  | [: `("", "<-");
       e2 = expr_1 ? P.error "expr" "'<-'" "[expr level expr1]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExAss loc e1 e2
  | [: :] -> a__ ]
and expr_4 =
  parser bp
    [: :] ->
      let a = expr_5 strm__ in
      expr_4_cont bp a strm__
and expr_4_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "or");
       e2 = P.orzero expr_4 expr_0 ? P.error "expr" "'or'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "or")) e1)
        e2
  | [: `("", "||");
       e2 = P.orzero expr_4 expr_0 ? P.error "expr" "'||'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "||")) e1)
        e2
  | [: :] -> a__ ]
and expr_5 =
  parser bp
    [: :] ->
      let a = expr_6 strm__ in
      expr_5_cont bp a strm__
and expr_5_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "&");
       e2 = P.orzero expr_5 expr_0 ? P.error "expr" "'&'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "&")) e1)
        e2
  | [: `("", "&&");
       e2 = P.orzero expr_5 expr_0 ? P.error "expr" "'&&'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "&&")) e1)
        e2
  | [: :] -> a__ ]
and expr_6 =
  parser bp
    [: :] ->
      let a = expr_7 strm__ in
      expr_6_cont bp a strm__
and expr_6_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "<");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'<'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "<")) e1) e2)
        strm__
  | [: `("", ">");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'>'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal ">")) e1) e2)
        strm__
  | [: `("", "<=");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'<='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "<=")) e1) e2)
        strm__
  | [: `("", ">=");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'>='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal ">=")) e1) e2)
        strm__
  | [: `("", "=");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "=")) e1) e2)
        strm__
  | [: `("", "<>");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'<>'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "<>")) e1) e2)
        strm__
  | [: `("", "==");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'=='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "==")) e1) e2)
        strm__
  | [: `("", "!=");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'!='" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "!=")) e1) e2)
        strm__
  | [: op = infixop0;
       e2 = P.orzero expr_7 expr_0
         ? P.error "expr" "[infixop0]" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_6_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal op)) e1) e2)
        strm__
  | [: :] -> a__ ]
and expr_7 =
  parser bp
    [: :] ->
      let a = expr_8 strm__ in
      expr_7_cont bp a strm__
and expr_7_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "^");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'^'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "^")) e1)
        e2
  | [: `("", "@");
       e2 = P.orzero expr_7 expr_0 ? P.error "expr" "'@'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "@")) e1)
        e2
  | [: op = infixop1;
       e2 = P.orzero expr_7 expr_0
         ? P.error "expr" "[infixop1]" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal op)) e1)
        e2
  | [: :] -> a__ ]
and expr_8 =
  parser bp
    [: :] ->
      let a = expr_9 strm__ in
      expr_8_cont bp a strm__
and expr_8_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "::");
       e2 = P.orzero expr_8 expr_0 ? P.error "expr" "'::'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExUid loc (Ploc.VaVal "::")) e1)
        e2
  | [: :] -> a__ ]
and expr_9 =
  parser bp
    [: :] ->
      let a = expr_10 strm__ in
      expr_9_cont bp a strm__
and expr_9_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "+");
       e2 = P.orzero expr_10 expr_0 ? P.error "expr" "'+'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_9_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "+")) e1) e2)
        strm__
  | [: `("", "-");
       e2 = P.orzero expr_10 expr_0 ? P.error "expr" "'-'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_9_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "-")) e1) e2)
        strm__
  | [: op = infixop2;
       e2 = P.orzero expr_10 expr_0
         ? P.error "expr" "[infixop2]" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_9_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal op)) e1) e2)
        strm__
  | [: :] -> a__ ]
and expr_10 =
  parser bp
    [: :] ->
      let a = expr_11 strm__ in
      expr_10_cont bp a strm__
and expr_10_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "*");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'*'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "*")) e1) e2)
        strm__
  | [: `("", "/");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'/'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "/")) e1) e2)
        strm__
  | [: `("", "%");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'%'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "%")) e1) e2)
        strm__
  | [: `("", "land");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'land'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "land")) e1) e2)
        strm__
  | [: `("", "lor");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'lor'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "lor")) e1) e2)
        strm__
  | [: `("", "lxor");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'lxor'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "lxor")) e1) e2)
        strm__
  | [: `("", "mod");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'mod'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "mod")) e1) e2)
        strm__
  | [: op = infixop3;
       e2 = P.orzero expr_11 expr_0
         ? P.error "expr" "[infixop3]" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_10_cont bp
        (MLast.ExApp loc
           (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal op)) e1) e2)
        strm__
  | [: :] -> a__ ]
and expr_11 =
  parser bp
    [: :] ->
      let a = expr_12 strm__ in
      expr_11_cont bp a strm__
and expr_11_cont bp a__ strm__ =
  match strm__ with parser
  [ [: `("", "**");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'**'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "**")) e1)
        e2
  | [: `("", "asr");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'asr'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc
        (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "asr")) e1) e2
  | [: `("", "lsl");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'lsl'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc
        (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "lsl")) e1) e2
  | [: `("", "lsr");
       e2 = P.orzero expr_11 expr_0 ? P.error "expr" "'lsr'" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc
        (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "lsr")) e1) e2
  | [: op = infixop4;
       e2 = P.orzero expr_11 expr_0
         ? P.error "expr" "[infixop4]" "[expr]" :] ->
      let e1 = a__ in
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal op)) e1)
        e2
  | [: :] -> a__ ]
and expr_12 =
  parser bp
  [ [: `("", "-");
       e = P.orzero expr_12 expr_0 ? P.error "expr" "'-'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "-")) e
  | [: `("", "-.");
       e = P.orzero expr_12 expr_0 ? P.error "expr" "'-.'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "-.")) e
  | [: :] -> expr_13 strm__ ]
and expr_13 =
  parser bp
  [ [: `("", "assert");
       e = P.orzero expr_13 expr_0 ? P.error "expr" "'assert'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      expr_13_cont bp (MLast.ExAsr loc e) strm__
  | [: `("", "lazy");
       e = P.orzero expr_13 expr_0 ? P.error "expr" "'lazy'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      expr_13_cont bp (MLast.ExLaz loc e) strm__
  | [: :] ->
      let a = expr_14 strm__ in
      expr_13_cont bp a strm__ ]
and expr_13_cont bp a__ strm__ =
  match try Some (expr_14 strm__) with [ Stream.Failure -> None ] with
  [ Some e2 ->
      let loc = P.gloc bp strm__ in
      let e1 = a__ in
      expr_13_cont bp
        (let (e1, e2) =
           if is_expr_constr_call e1 then
             match e1 with
             [ MLast.ExApp _ e11 e12 -> (e11, MLast.ExApp loc e12 e2)
             | _ -> (e1, e2) ]
           else (e1, e2)
         in
         match constr_expr_arity loc e1 with
         [ 1 -> MLast.ExApp loc e1 e2
         | _ ->
             match e2 with
             [ MLast.ExTup _ (Ploc.VaVal el) ->
                 List.fold_left (fun e1 e2 -> MLast.ExApp loc e1 e2) e1 el
             | _ -> MLast.ExApp loc e1 e2 ] ])
        strm__
  | _ -> a__ ]
and expr_14 =
  parser bp
  [ [: i = P.vala (P.token ("TILDEIDENTCOLON", ""));
       e = P.orzero expr_14 expr_0
         ? P.error "expr" "TILDEIDENTCOLON" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExLab loc (Ploc.VaVal [(MLast.PaLid loc i, Ploc.VaVal (Some e))])
  | [: i = P.vala (P.token ("TILDEIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExLab loc (Ploc.VaVal [(MLast.PaLid loc i, Ploc.VaVal None)])
  | [: i = P.vala (P.token ("QUESTIONIDENTCOLON", ""));
       e = P.orzero expr_14 expr_0
         ? P.error "expr" "QUESTIONIDENTCOLON" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExOlb loc (MLast.PaLid loc i) (Ploc.VaVal (Some e))
  | [: i = P.vala (P.token ("QUESTIONIDENT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExOlb loc (MLast.PaLid loc i) (Ploc.VaVal None)
  | [: :] -> expr_15 strm__ ]
and expr_15 =
  parser bp
    [: :] ->
      let a = expr_16 strm__ in
      expr_15_cont bp a strm__
and expr_15_cont bp a__ strm__ =
  match Stream.npeek 2 strm__ with
  [ [("", "#"); ("LIDENT", lab)] -> do {
      Stream.junk strm__;
      Stream.junk strm__;
      let lab = Ploc.VaVal lab in
      try
        let loc = P.gloc bp strm__ in
        let e = a__ in
        expr_15_cont bp (MLast.ExSnd loc e lab) strm__
      with
      [ Stream.Failure -> raise (Stream.Error (P.error "expr" "'#'" "???")) ]
    }
  | _ ->
      match strm__ with parser
      [ [: `("", ".") :] ->
          try
            match strm__ with parser
            [ [: `("", "(") :] ->
                try
                  match
                    try Some (operator_rparen strm__) with
                    [ Stream.Failure -> None ]
                  with
                  [ Some op ->
                      let loc = P.gloc bp strm__ in
                      let e1 = a__ in
                      expr_15_cont bp
                        (MLast.ExAre loc e1 (MLast.ExLid loc (Ploc.VaVal op)))
                        strm__
                  | _ ->
                      match
                        try Some (expr_0 strm__) with
                        [ Stream.Failure -> None ]
                      with
                      [ Some e2 ->
                          match strm__ with parser
                          [ [: `("", ")") :] ->
                              let loc = P.gloc bp strm__ in
                              let e1 = a__ in
                              expr_15_cont bp (MLast.ExAre loc e1 e2) strm__
                          | [: :] ->
                              raise
                                (Stream.Error
                                   (P.error "expr" "[expr]" "')'")) ]
                      | _ -> raise Stream.Failure ] ]
                with
                [ Stream.Failure ->
                    raise
                      (Stream.Error
                         (P.error "expr" "'('"
                            "[operator_rparen] or [expr]")) ]
            | [: `("", "["); e2 = expr_0 ? P.error "expr" "'['" "[expr]";
                 `("", "]") ? P.error "expr" "[expr]" "']'" :] ->
                let loc = P.gloc bp strm__ in
                let e1 = a__ in
                expr_15_cont bp (MLast.ExSte loc e1 e2) strm__
            | [: `("", "{");
                 el = P.vala (P.list1sep expr_9 (P.token ("", ",")) False)
                   ? P.error "expr" "'{'" "???";
                 `("", "}") ? P.error "expr" "???" "'}'" :] ->
                let loc = P.gloc bp strm__ in
                let e = a__ in
                expr_15_cont bp (MLast.ExBae loc e el) strm__
            | [: e2 = expr_16 :] ->
                let loc = P.gloc bp strm__ in
                let e1 = a__ in
                expr_15_cont bp
                  (loop e1 e2 where rec loop m =
                     fun
                     [ MLast.ExAcc _ x y -> loop (MLast.ExAcc loc m x) y
                     | e -> MLast.ExAcc loc m e ])
                  strm__ ]
          with
          [ Stream.Failure ->
              raise
                (Stream.Error
                   (P.error "expr" "'.'" "'(' or '[' or '{' or [expr]")) ]
      | [: :] -> a__ ] ]
and expr_16 =
  parser bp
  [ [: `("", "!");
       e = P.orzero expr_16 expr_0 ? P.error "expr" "'!'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExAcc loc e (MLast.ExLid loc (Ploc.VaVal "val"))
  | [: `("", "~-");
       e = P.orzero expr_16 expr_0 ? P.error "expr" "'~-'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "~-")) e
  | [: `("", "~-.");
       e = P.orzero expr_16 expr_0 ? P.error "expr" "'~-.'" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal "~-.")) e
  | [: f = prefixop;
       e = P.orzero expr_16 expr_0
         ? P.error "expr" "[prefixop]" "[expr]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExApp loc (MLast.ExLid loc (Ploc.VaVal f)) e
  | [: :] -> expr_17 strm__ ]
and expr_17 =
  parser bp
  [ [: `("", "[<") :] ->
      try
        match strm__ with parser
        [ [: `("", ">]") :] ->
            let loc = P.gloc bp strm__ in
            cstream loc []
        | [: sel = stream_expr_comp_list_0;
             `("", ">]")
               ? P.error "expr" "[stream_expr_comp_list]" "'>]'" :] ->
            let loc = P.gloc bp strm__ in
            cstream loc sel ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'[<'" "'>]' or [stream_expr_comp_list]")) ]
  | [: `("", "`"); s = P.vala ident_0 ? P.error "expr" "'`'" "[ident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExVrn loc s
  | [: `("", "{<") :] ->
      try
        match strm__ with parser
        [ [: `("", ">}") :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExOvr loc (Ploc.VaVal [])
        | [: fel = P.vala field_expr_list_0;
             `("", ">}") ? P.error "expr" "[field_expr_list]" "'>}'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExOvr loc fel ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'{<'" "'>}' or [field_expr_list]")) ]
  | [: `("", "new");
       i = P.vala class_longident_0
         ? P.error "expr" "'new'" "[class_longident]" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExNew loc i
  | [: `("", "object");
       cspo = P.vala (P.option class_self_patt_0)
         ? P.error "expr" "'object'" "???";
       cf = P.vala class_structure_0
         ? P.error "expr" "???" "[class_structure]";
       `("", "end") ? P.error "expr" "[class_structure]" "'end'" :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExObj loc cspo cf
  | [: `("UIDENT", "True") :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExUid loc (Ploc.VaVal "True_")
  | [: `("UIDENT", "False") :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExUid loc (Ploc.VaVal "False_")
  | [: `("", "false") :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExUid loc (Ploc.VaVal "False")
  | [: `("", "true") :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExUid loc (Ploc.VaVal "True")
  | [: `("", "[") :] ->
      try
        match strm__ with parser
        [ [: `("", "]") :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExUid loc (Ploc.VaVal "[]")
        | [: el = expr1_semi_list_0;
             `("", "]") ? P.error "expr" "[expr1_semi_list]" "']'" :] ->
            let loc = P.gloc bp strm__ in
            mklistexp loc None el ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error (P.error "expr" "'['" "']' or [expr1_semi_list]")) ]
  | [: `("", "[|") :] ->
      try
        match strm__ with parser
        [ [: `("", "|]") :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExArr loc (Ploc.VaVal [])
        | [: el = P.vala expr1_semi_list_0;
             `("", "|]") ? P.error "expr" "[expr1_semi_list]" "'|]'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExArr loc el ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'[|'" "'|]' or [expr1_semi_list]")) ]
  | [: `("", "{") :] ->
      try
        match
          try Some (test_label_eq strm__) with [ Stream.Failure -> None ]
        with
        [ Some _ ->
            match
              try Some (P.vala lbl_expr_list_0 strm__) with
              [ Stream.Failure -> None ]
            with
            [ Some lel ->
                match strm__ with parser
                [ [: `("", "}") :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.ExRec loc lel None
                | [: :] ->
                    raise
                      (Stream.Error
                         (P.error "expr" "[lbl_expr_list]" "'}'")) ]
            | _ ->
                raise
                  (Stream.Error
                     (P.error "expr" "[test_label_eq]" "[lbl_expr_list]")) ]
        | _ ->
            match
              try Some (expr_13 strm__) with [ Stream.Failure -> None ]
            with
            [ Some e ->
                match strm__ with parser
                [ [: `("", "with");
                     lel = P.vala lbl_expr_list_0
                       ? P.error "expr" "'with'" "[lbl_expr_list]";
                     `("", "}") ? P.error "expr" "[lbl_expr_list]" "'}'" :] ->
                    let loc = P.gloc bp strm__ in
                    MLast.ExRec loc lel (Some e)
                | [: :] ->
                    raise
                      (Stream.Error
                         (P.error "expr" "[expr level apply]" "'with'")) ]
            | _ -> raise Stream.Failure ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'{'"
                  "[test_label_eq] or [expr level apply]")) ]
  | [: `("", "(") :] ->
      try
        match strm__ with parser
        [ [: `("", ")") :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExUid loc (Ploc.VaVal "()")
        | [: `("", "module");
             me = module_expr_0
               ? P.error "expr" "'module'" "[module_expr]" :] ->
            try
              match strm__ with parser
              [ [: `("", ":");
                   mt = module_type_0 ? P.error "expr" "':'" "[module_type]";
                   `("", ")") ? P.error "expr" "[module_type]" "')'" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.ExPck loc me (Some mt)
              | [: `("", ")") :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.ExPck loc me None ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "expr" "[module_expr]" "':' or ')'")) ]
        | [: op = operator_rparen :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExLid loc (Ploc.VaVal op)
        | [: el = P.vala e_phony;
             `("", ")") ? P.error "expr" "[e_phony]" "')'" :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExTup loc el
        | [: e = expr_0 :] ->
            try
              match strm__ with parser
              [ [: `("", ":>"); t = ctyp_0 ? P.error "expr" "':>'" "[ctyp]";
                   `("", ")") ? P.error "expr" "[ctyp]" "')'" :] ->
                  let loc = P.gloc bp strm__ in
                  MLast.ExCoe loc e None t
              | [: `("", ":");
                   t = ctyp_0 ? P.error "expr" "':'" "[ctyp]" :] ->
                  try
                    match strm__ with parser
                    [ [: `("", ":>");
                         t2 = ctyp_0 ? P.error "expr" "':>'" "[ctyp]";
                         `("", ")") ? P.error "expr" "[ctyp]" "')'" :] ->
                        let loc = P.gloc bp strm__ in
                        MLast.ExCoe loc e (Some t) t2
                    | [: `("", ")") :] ->
                        let loc = P.gloc bp strm__ in
                        MLast.ExTyc loc e t ]
                  with
                  [ Stream.Failure ->
                      raise
                        (Stream.Error
                           (P.error "expr" "[ctyp]" "':>' or ')'")) ]
              | [: `("", ")") :] ->
                  let loc = P.gloc bp strm__ in
                  concat_comm loc e ]
            with
            [ Stream.Failure ->
                raise
                  (Stream.Error
                     (P.error "expr" "[expr]" "':>' or ':' or ')'")) ] ]
      with
      [ Stream.Failure ->
          raise
            (Stream.Error
               (P.error "expr" "'('"
                  "')' or 'module' or [operator_rparen] or [e_phony] or [expr]")) ]
  | [: `("", "begin") :] ->
      try
        match strm__ with parser
        [ [: `("", "end") :] ->
            let loc = P.gloc bp strm__ in
            MLast.ExUid loc (Ploc.VaVal "()")
        | [: e = expr_0; `("", "end") ? P.error "expr" "[expr]" "'end'" :] ->
            let loc = P.gloc bp strm__ in
            concat_comm loc e ]
      with
      [ Stream.Failure ->
          raise (Stream.Error (P.error "expr" "'begin'" "'end' or [expr]")) ]
  | [: s = P.vala (P.token ("INT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExInt loc s ""
  | [: s = P.vala (P.token ("INT_l", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExInt loc s "l"
  | [: s = P.vala (P.token ("INT_L", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExInt loc s "L"
  | [: s = P.vala (P.token ("INT_n", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExInt loc s "n"
  | [: s = P.vala (P.token ("FLOAT", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExFlo loc s
  | [: s = P.vala (P.token ("STRING", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExStr loc s
  | [: c = P.vala (P.token ("CHAR", "")) :] ->
      let loc = P.gloc bp strm__ in
      MLast.ExChr loc c
  | [: `("QUOTATION", x) :] ->
      let loc = P.gloc bp strm__ in
      let con = quotation_content x in
      Pcaml.handle_expr_quotation loc con
  | [: i = expr_ident_0 :] -> i ]
and expr_gen_cont bp a__ strm__ =
  let x = expr_0_cont bp a__ strm__ in
  let x = expr_2_cont bp x strm__ in
  let x = expr_3_cont bp x strm__ in
  let x = expr_4_cont bp x strm__ in
  let x = expr_5_cont bp x strm__ in
  let x = expr_6_cont bp x strm__ in
  let x = expr_7_cont bp x strm__ in
  let x = expr_8_cont bp x strm__ in
  let x = expr_9_cont bp x strm__ in
  let x = expr_10_cont bp x strm__ in
  let x = expr_11_cont bp x strm__ in
  let x = expr_13_cont bp x strm__ in
  let x = expr_15_cont bp x strm__ in
  if x == a__ then x else expr_gen_cont bp x strm__
and str_item_semi_0 =
  parser bp
    [: si = str_item_0 :] ->
      let _ =
        try P.option (P.token ("", ";;")) strm__ with
        [ Stream.Failure -> raise (Stream.Error "") ]
      in
      let loc = P.gloc bp strm__ in
      (si, loc)
;

let lexer = Lazy.force P.lexer in
List.iter (fun kw -> lexer.Plexing.tok_using ("", kw))
  ["!"; "!="; "#"; "%"; "&"; "&&"; "'"; "("; ")"; "*"; "**"; "+"; ","; "-";
   "-."; "->"; "."; ".."; "/"; ":"; "::"; ":="; ":>"; ";"; ";;"; "<"; "<-";
   "<="; "<>"; "="; "=="; ">"; ">="; ">]"; ">}"; "?"; "?!"; "?="; "??"; "@";
   "["; "[<"; "[|"; "]"; "^"; "_"; "`"; "and"; "as"; "asr"; "assert"; "begin";
   "class"; "constraint"; "do"; "done"; "downto"; "else"; "end"; "exception";
   "external"; "false"; "for"; "fun"; "function"; "functor"; "if"; "in";
   "include"; "inherit"; "initializer"; "land"; "lazy"; "let"; "lor"; "lsl";
   "lsr"; "lxor"; "match"; "method"; "mod"; "module"; "mutable"; "new";
   "nonrec"; "object"; "of"; "open"; "or"; "parser"; "private"; "rec"; "sig";
   "struct"; "then"; "to"; "true"; "try"; "type"; "val"; "virtual"; "when";
   "while"; "with"; "{"; "{<"; "|"; "|]"; "||"; "}"; "~"; "~-"; "~-."];
(****************************************)

value interf_p =
  Grammar.Entry.of_parser Pcaml.gram "interf" interf_0
;

value implem_p =
  Grammar.Entry.of_parser Pcaml.gram "implem" implem_0
;

value top_phrase_p =
  Grammar.Entry.of_parser Pcaml.gram "top_phrase" top_phrase_0
;

value use_file_p =
  Grammar.Entry.of_parser Pcaml.gram "use_file" use_file_0
;

EXTEND
  interf:
    [ [ x = interf_p -> x ] ]
  ;
  implem:
    [ [ x = implem_p -> x ] ]
  ;
  top_phrase:
    [ [ x = top_phrase_p -> x ] ]
  ;
  use_file:
    [ [ x = use_file_p -> x ] ]
  ;
END;
